package com.ut.mini;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import android.net.Uri;

import com.ut.mini.base.UTMIVariables;
import com.ut.mini.internal.UTOriginalCustomHitBuilder;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * 
 * Fix（2014-11-29）:代码中用了String.isEmpty这个函数，这个函数要求9到以上的sdk版本才可以使用。MiniSDK的要求是4版本
 * 代码中，我用正则匹配了内容：([0-9a-zA-Z_-]*).isEmpty\(\) ，用如下正则进行替换：UTMCStringUtils.isEmpty(\1)
 *
 */
public class UTHybridHelper {
	
	private static UTHybridHelper s_instance = new UTHybridHelper();

	/**
	 * 获取UTHybridHelper实例
	 * @return
	 */
	public static UTHybridHelper getInstance() {
		return s_instance;
	}
	
    public void setH5Url(String url){
    	if(url != null)
    	{
    		UTMIVariables.getInstance().setH5Url(url);
    	}
    }
    
	public void h5UT(Map<String,String> dataMap, Object view) {
		if (dataMap == null || dataMap.size() == 0) {
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5UT",
                    "dataMap is empty");
            return;
		}
		
		String funcType = dataMap.get("functype");
			
		if(funcType == null){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5UT","funcType is null");
			return;
		}
		String lutjstype = dataMap.get("utjstype");
		if(lutjstype != null && !lutjstype.equals("0") && !lutjstype.equals("1")){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5UT","utjstype should be 1 or 0 or null");
			return;
		}
		
		dataMap.remove("functype");
		Date opTime = new Date();
		if(funcType.equals("2001")){
			h5Page(opTime,dataMap,view);
		}else if(funcType.equals("2101")){
			h5Ctrl(opTime,dataMap);
		}

	}
	
    private void h5Page(Date opTime,Map<String,String> dataMap,Object webview){
    	if(dataMap == null || dataMap.size() == 0){
    		return;
    	}
    		
        String urlPageName = dataMap.get("urlpagename");
        String url = dataMap.get("url");
        String pageName = getH5PageName(urlPageName,url);
        
        //到这步pageName还为null 则返回
        if(pageName == null || UTMCStringUtils.isEmpty(pageName)){
            UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5Page",
                    "pageName is null,return");
            return;
        }
		
      //  String prePageName = UTMIVariables.getInstance().getmH5RefPage();
          String prePageName = UTMIVariables.getInstance().getRefPage();

        Map<String, String> args = null;
        String utjstype = dataMap.get("utjstype");
        dataMap.remove("utjstype");
        if(utjstype == null || utjstype.equals("0") ){
        	args = h5PageParseArgsWithAplus(dataMap);
        }else if(utjstype.equals("1")){
        	args = h5PageParseArgsWithOutAplus(dataMap);
        }
        
        /**
         * Add by daima[2015年 6月 5日 星期五 14时17分42秒 CST][start]
         */
        int lEventID = 2006;
        if(UTPageHitHelper.getInstance().isH52001(webview)){
        	lEventID = 2001;
        }
        
        UTOriginalCustomHitBuilder lHitBuilder = new UTOriginalCustomHitBuilder(pageName,lEventID,prePageName,null,null,args);
        
        if(2001 == lEventID){
			UTMIVariables.getInstance().setRefPage(pageName);
		}
        
        Map<String,String> lNextPageProperties = UTPageHitHelper.getInstance().getAndResetNextPageProperties();
        if(null != lNextPageProperties && lNextPageProperties.size()>0){
        	lHitBuilder.setProperties(lNextPageProperties);
        }
        
        /**
         * Add by daima[2015年 6月 5日 星期五 14时17分42秒 CST][end]
         */
        
		UTTracker lTracker = UTAnalytics.getInstance().getDefaultTracker();
		if(null != lTracker){
			lTracker.send(lHitBuilder.build());
//			UTMIVariables.getInstance().setRefPage(pageName);
//			UTMIVariables.getInstance().setH5PageCalled(true);
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5Page event error", "Fatal Error,must call setRequestAuthentication method first.");
		}
		
		UTPageHitHelper.getInstance().setH5Called(webview);
    	
    }
    
    private void h5Ctrl(Date opTime,Map<String,String> dataMap){
    	if(dataMap == null || dataMap.size() == 0){
    		return;
    	}
        String urlPageName = dataMap.get("urlpagename");
        String url = dataMap.get("url");
        String pageName = getH5PageName(urlPageName,url);
			
        if(pageName == null || UTMCStringUtils.isEmpty(pageName)){
        	UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "h5Ctrl", "pageName is null,return");
        	return;
        }
        
        String logKey = dataMap.get("logkey");
        
        if(logKey == null || UTMCStringUtils.isEmpty(logKey)){
        	UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "h5Ctrl", "logkey is null,return");
        	return;
        }

        Map<String, String> args = null;
        String utjstype = dataMap.get("utjstype");
        dataMap.remove("utjstype");
        if(utjstype == null || utjstype.equals("0") ){
        	args = h5CtrlParseArgsWithAplus(dataMap);
        }else if(utjstype.equals("1")){
        	args = h5CtrlParseArgsWithOutAplus(dataMap);
        }       
        

        UTOriginalCustomHitBuilder lOchb = new UTOriginalCustomHitBuilder(pageName,2101,logKey,null,null,args);
		UTTracker lTracker = UTAnalytics.getInstance().getDefaultTracker();
		if(null != lTracker){
			lTracker.send(lOchb.build());
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "h5Ctrl event error", "Fatal Error,must call setRequestAuthentication method first.");
		}
    }
    
    
    private Map<String, String> h5PageParseArgsWithAplus(Map<String,String> dataMap){
    	if(dataMap == null || dataMap.size() == 0){
    		return null;
    	}
    	
    	HashMap<String,String> lArgs = new HashMap<String,String>();
    	String url = dataMap.get("url");

    	lArgs.put("_h5url", url == null?"":url);

    	if(null != url){
	    	Uri lUrl = Uri.parse(url);
	    	String 	spm = lUrl.getQueryParameter("spm");
	    	if(spm != null && !UTMCStringUtils.isEmpty(spm)){
	    		lArgs.put("spm", spm);
	    	}else{
	    		lArgs.put("spm", "0.0.0.0");
	    	}
	    	
	    	String 	scm = lUrl.getQueryParameter("scm");
	    	if(scm != null && !UTMCStringUtils.isEmpty(scm)){
	    		lArgs.put("scm", scm);
	    	}
    	}else{
    		lArgs.put("spm", "0.0.0.0");
    	}
    	String spmcnt = dataMap.get("spmcnt");
    	lArgs.put("_spmcnt", spmcnt == null?"":spmcnt);
    	
    	String spmpre = dataMap.get("spmpre");
    	lArgs.put("_spmpre", spmpre == null?"":spmpre);
    	
    	String lzsid = dataMap.get("lzsid");
    	lArgs.put("_lzsid", lzsid == null?"":lzsid);
    	
    	String eargs = dataMap.get("extendargs");
    	lArgs.put("_h5ea", eargs == null?"":eargs);
    	
    	String cna = dataMap.get("cna");
    	lArgs.put("_cna", cna == null?"":cna);
    	
    	lArgs.put("_ish5", "1");
    	
    	return lArgs;
}
    
    private Map<String, String> h5PageParseArgsWithOutAplus(Map<String,String> dataMap){
    	if(dataMap == null || dataMap.size() == 0){
    		return null;
    	}
    	
    	HashMap<String,String> lArgs = new HashMap<String,String>();
    	String url = dataMap.get("url");
    	lArgs.put("_h5url", url == null?"":url);
    	
    	String extendargs = dataMap.get("extendargs");
    	lArgs.put("_h5ea", extendargs == null?"":extendargs);    	
    	lArgs.put("_ish5", "1");

    	return lArgs;
    }
    
    private Map<String, String> h5CtrlParseArgsWithAplus(Map<String,String> dataMap){
    	if(dataMap == null || dataMap.size() == 0){
    		return null;
    	}
    	
    	Map<String, String> lArgs = new HashMap<String, String>();
    	String logkeyargs = dataMap.get("logkeyargs");
    	lArgs.put("_lka", logkeyargs == null?"":logkeyargs);
    	
    	String cna = dataMap.get("cna");
    	lArgs.put("_cna", cna == null?"":cna);
    	
    	String extendargs = dataMap.get("extendargs");
    	lArgs.put("_h5ea", extendargs == null?"":extendargs);
    	
    	lArgs.put("_ish5", "1");
    	
    	return lArgs;
    }
    
    private Map<String, String> h5CtrlParseArgsWithOutAplus(Map<String,String> dataMap){
    	if(dataMap == null || dataMap.size() == 0){
    		return null;
    	}
    	
    	Map<String, String> lArgs = new HashMap<String, String>();
    	
    	String extendargs = dataMap.get("extendargs");
    	lArgs.put("_h5ea", extendargs == null?"":extendargs);    	
    	lArgs.put("_ish5", "1");
    	
    	return lArgs;
    }
    private String getH5PageName(String urlPageName,String url){
    	//只要urlPageName有值就取urlPageName，为空则取url问号前面的
    	String pageName = "";
    	if(urlPageName != null && !UTMCStringUtils.isEmpty(urlPageName) /*&&  isOnePage != null && isOnePage.equals("1")*/){
        	pageName = urlPageName;
        }else{
        	if(!UTMCStringUtils.isEmpty(url)){
        		int pos = url.indexOf("?");
        		if(pos == -1){//没有问号的 直接取url
        			pageName = url;
        		}else{
	        		pageName = url.substring(0, pos);
        		}
        	}
        }
    	return pageName;
    }
}
