package com.ut.mini.base;

import java.io.UnsupportedEncodingException;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Build;
import android.util.Log;

import com.ut.mini.UTAnalytics;
import com.ut.mini.core.UTMCLogTransferMain;
import com.ut.mini.core.appstatus.UTMCAppStatusRegHelper;
import com.ut.mini.core.sign.IUTRequestAuthentication;
import com.ut.mini.info.UTMCBuildInfo;
import com.ut.mini.internal.UTOriginalCustomHitBuilder;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCBase64;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * 静态配置类
 * 
 * @author daichenjie 针对全局要对外提供访问的配置性成员，可以添加到这个类中 独立了一个包名 com.ut.mini.base
 *         方便做到和对外的隔离
 * @update 移除setKey接口，改用IRequestAuthentication来代替。
 */
public class UTMCStatConfig {

	private static UTMCStatConfig s_instance = new UTMCStatConfig();

	private UTMCStatConfig() {

	}

	public static UTMCStatConfig getInstance() {
		return s_instance;
	}

	private Context mContext = null;
	private String mAppkey = null;
	private String mChannel = null;
	// 这两个ID必然是共享的
	private String mUsernick = null;// 用户昵称
	private String mLUsernick = null;
	private String mUserid = null;// 用户ID,这个ID 还需要持久，尼玛很坑爹啊
	private String mLUserid = null;
	private boolean mIsSecuritySDKSupportOn = false;
	private Application mApplication = null;
	private String mAppVersion = null;
	private IUTRequestAuthentication mRequestAuthenticationInstance = null;
	private boolean mRegisterActivityLifecircleCallbacks = false;
	private IUTMCBuildInfo mBuildInfo = null;

	public void setAppVersion(String aAppVersion) {
		mAppVersion = aAppVersion;
	}

	public String getAppVersion() {
		return mAppVersion;
	}
	
	public IUTMCBuildInfo getBuildInfo(){
		return UTMCBuildInfo.getInstance();
	}

	public void setRequestAuthenticationInstance(
			IUTRequestAuthentication aRequestAuthenticationInstance) {
		mRequestAuthenticationInstance = aRequestAuthenticationInstance;
		if (null != aRequestAuthenticationInstance) {
			this.mAppkey = aRequestAuthenticationInstance.getAppkey();
		}
		// BugFix:戴码 20150329 下午5点
		// 如果没有这句话，会导致，setContext在这个方法前面调用的时候，在没有任何打点的情况下，不会自动触发数据传输逻辑
		_ActiveTransder();
	}

	public IUTRequestAuthentication getRequestAuthenticationInstance() {
		return mRequestAuthenticationInstance;
	}

	public void turnOnDebug() {
		UTMCLogger.setDebug(true);
	}

	public void setChannel(String aChannel) {
		mChannel = aChannel;
	}

	public String getLongLoginUsernick() {
		return mLUsernick;
	}

	public String getLongLoingUserid() {
		return mLUserid;
	}

	public String getChannel() {
		return mChannel;
	}

	private void _setUsernick(String pUsernick) {
		mUsernick = pUsernick;
		if (!UTMCStringUtils.isEmpty(pUsernick)) {
			mLUsernick = pUsernick;
		}
		if (!UTMCStringUtils.isEmpty(pUsernick) && null != mContext) {
			try {
				SharedPreferences lSP = mContext.getSharedPreferences(
						"UTCommon", Context.MODE_PRIVATE);
				Editor lE = lSP.edit();
				lE.putString(
						"_lun",
						new String(
								UTMCBase64.encode(pUsernick.getBytes("UTF-8"),
										UTMCBase64.NO_WRAP)));
				lE.commit();
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
	}

	public String getUsernick() {
		return mUsernick;
	}

	private void _setUserid(String pUserid) {
		mUserid = pUserid;
		if (!UTMCStringUtils.isEmpty(pUserid)) {
			mLUserid = pUserid;
		}
		if (!UTMCStringUtils.isEmpty(pUserid) && null != mContext) {
			try {
				SharedPreferences lSP = mContext.getSharedPreferences(
						"UTCommon", Context.MODE_PRIVATE);
				Editor lE = lSP.edit();
				lE.putString(
						"_luid",
						new String(UTMCBase64.encode(pUserid.getBytes("UTF-8"),
								UTMCBase64.NO_WRAP)));
				lE.commit();
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
	}

	public String getUserid() {
		return mUserid;
	}

	public void updateUserAccount(String aUsernick, String aUserid) {
		_setUsernick(aUsernick);
		_setUserid(aUserid);
		// 如果用户昵称不为空，那么我们发送一个1007事件
		if(!UTMCStringUtils.isEmpty(aUsernick)){
			UTOriginalCustomHitBuilder lBuilder = new UTOriginalCustomHitBuilder("UT",1007,aUsernick,aUserid,null,null);
			UTAnalytics.getInstance().getDefaultTracker().send(lBuilder.build());
		}
	}

	public void setContext(Context aContext) {
		if (null != aContext) {
			mContext = aContext;
			SharedPreferences lSP = mContext.getSharedPreferences("UTCommon",
					Context.MODE_PRIVATE);
			String lUN = lSP.getString("_lun", "");
			if (!UTMCStringUtils.isEmpty(lUN)) {
				String lUserNick;
				try {
					lUserNick = new String(UTMCBase64.decode(lUN.getBytes(),
							UTMCBase64.NO_WRAP), "UTF-8");
					this.mLUsernick = lUserNick;
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
			String lUid = lSP.getString("_luid", "");
			if (!UTMCStringUtils.isEmpty(lUid)) {
				String lUserid;
				try {
					lUserid = new String(UTMCBase64.decode(lUid.getBytes(),
							UTMCBase64.NO_WRAP), "UTF-8");
					this.mLUserid = lUserid;
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
		}
		_regActivityLifecircleCallbacks();
		_ActiveTransder();
	}

	public Context getContext() {
		return mContext;
	}

	private void _ActiveTransder() {
		if (null != mAppkey && mContext != null) {
			// 打一个空点，激活通道
			UTMCLogTransferMain.getInstance().transferLog(null);
		}
	}

	public String getAppkey() {
		return mAppkey;
	}

	public boolean isSecuritySDKSupportOn() {
		return mIsSecuritySDKSupportOn;
	}

	public void turnOnSecuritySDKSupport(String aAppKey) {
		mIsSecuritySDKSupportOn = true;
		mAppkey = aAppKey;
	}

	public void setAppApplicationInstance(Application aApplicationInstance) {
		this.mApplication = aApplicationInstance;
		_regActivityLifecircleCallbacks();
	}

	/**
	 * 4.0以上注册应用activity切换通知
	 */
	private void _regActivityLifecircleCallbacks() {
		// application context:注册句柄
		if (!mRegisterActivityLifecircleCallbacks) {
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
				// 注册一个 activity的活动回调
				try {
					// 如果外部设置了一个application的实例，那么我们就用它来注册，否则，我们用context的转换来设置
					if (null != UTMCStatConfig.getInstance()
							.getAppApplicationInstance()) {
						Application lApplicationInstance = UTMCStatConfig
								.getInstance().getAppApplicationInstance();
						UTMCAppStatusRegHelper
								.registeActivityLifecycleCallbacks(lApplicationInstance);
						mRegisterActivityLifecircleCallbacks = true;
					} else {
						Context lContext = UTMCStatConfig.getInstance()
								.getContext().getApplicationContext();
						UTMCAppStatusRegHelper
								.registeActivityLifecycleCallbacks((Application) lContext);
						mRegisterActivityLifecircleCallbacks = true;
					}
				} catch (Exception e) {
					e.printStackTrace();
					Log.e("UTEngine",
							"You need set a application instance for UT.");
				}
			}
		}
	}

	public Application getAppApplicationInstance() {
		return this.mApplication;
	}
	
    // 存储一个设备是否是一个旧设备，这个标记:这个方法存放在这里不是很合适
    public void setIsOldDevice(boolean old) {
    	if (mContext != null) {
    		SharedPreferences sp = mContext.getSharedPreferences("UTCommon", Context.MODE_PRIVATE);
    		sp.edit().putBoolean("_isolddevice", old).commit();
    	}
    }

    private boolean mIsOldDevice = false;
    // 存储一个设备是否是一个旧设备，这个标记:这个方法存放在这里不是很合适
    public boolean isOldDevice() {
    	if (!mIsOldDevice && mContext != null) {
    		SharedPreferences sp = mContext.getSharedPreferences("UTCommon", Context.MODE_PRIVATE);
    		mIsOldDevice = sp.getBoolean("_isolddevice", false);
    	}
    	return mIsOldDevice;
    }

}
