
package com.ut.mini.core;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Build;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.util.Log;

import com.ta.utdid2.device.UTDevice;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCLogFields;
import com.ut.mini.utils.SystemProperties;
import com.ut.mini.utils.UTMCNetworkUtils;
import com.ut.mini.utils.UTMCPhoneInfoUtils;
import com.ut.mini.utils.UTMCStringUtils;


public class UTMCDevice {

    private final static String TAG = "UTMCDevice";
    private static Map<String, String> s_deviceInfoMap = null;
    
    public synchronized static void updateNetworkStatus(Context aContext){
    	if(null != s_deviceInfoMap){
	    	try {
	            String[] networkStatus = UTMCNetworkUtils
	                    .getNetworkState(aContext);
	            s_deviceInfoMap.put(UTLogFieldsScheme.ACCESS.toString(), networkStatus[0]);
	            if (networkStatus[0].equals("2G/3G"))
	            	s_deviceInfoMap.put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(),
	                        networkStatus[1]);
	            else {
	            	s_deviceInfoMap.put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(),
	                        "Unknown");
	            }
	        } catch (Exception e) {
	        	s_deviceInfoMap.put(UTLogFieldsScheme.ACCESS.toString(), "Unknown");
	        	s_deviceInfoMap
	                    .put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(), "Unknown");
	        }
    	}
    }

    public static synchronized Map<String, String> getDeviceInfo(Context pContext) {
        if (s_deviceInfoMap != null) {
            return s_deviceInfoMap;
        }
        if (pContext != null) {
            Map<String, String> lDeviceInfoMap = new HashMap<String, String>();
            if (lDeviceInfoMap != null) {
            	
                try {
                    // 调用UTDID
                    try {
                        lDeviceInfoMap.put(UTLogFieldsScheme.UTDID.toString(),
                                UTDevice.getUtdid(pContext));
                    } catch (Exception e) {
                        Log.e(TAG, "utdid4all jar doesn't exist, please copy the libs folder.");
                        e.printStackTrace();
                    }

                    lDeviceInfoMap.put(UTLogFieldsScheme.IMEI.toString(),
                            UTMCPhoneInfoUtils.getImei(pContext));
                    lDeviceInfoMap.put(UTLogFieldsScheme.IMSI.toString(),
                            UTMCPhoneInfoUtils.getImsi(pContext));
                    
                    lDeviceInfoMap.put(UTLogFieldsScheme.DEVICE_MODEL.toString(), Build.MODEL);
                    lDeviceInfoMap.put(UTLogFieldsScheme.BRAND.toString(), Build.BRAND);
                    lDeviceInfoMap.put(UTLogFieldsScheme.OSVERSION.toString(),
                            Build.VERSION.RELEASE);
                    lDeviceInfoMap.put(UTLogFieldsScheme.OS.toString(), "a");

                    try {
                        PackageInfo packageInfo = pContext.getPackageManager()
                                .getPackageInfo(pContext.getPackageName(), 0);
                        lDeviceInfoMap.put(UTLogFieldsScheme.APPVERSION.toString(),
                                packageInfo.versionName);
                    } catch (PackageManager.NameNotFoundException e) {
                        lDeviceInfoMap.put(UTLogFieldsScheme.APPVERSION.toString(), "Unknown");
                    }

                    if (isYunOSSystem()) {
                        lDeviceInfoMap.put(UTLogFieldsScheme.OS.toString(), "y");
                        String lUUID = getUUID();
                        if (!UTMCStringUtils.isEmpty(lUUID)) {
                            lDeviceInfoMap.put(UTMCLogFields.DEVICE_ID.toString(), lUUID);
                        }
                        String lOSVersion = System.getProperty("ro.yunos.version");
                        if (!UTMCStringUtils.isEmpty(lOSVersion)) {
                            lDeviceInfoMap
                                    .put(UTLogFieldsScheme.OSVERSION.toString(), lOSVersion);
                        }
                        lOSVersion = _getBuildVersion();
                        if (!UTMCStringUtils.isEmpty(lOSVersion)) {
                            lDeviceInfoMap
                                    .put(UTLogFieldsScheme.OSVERSION.toString(), lOSVersion);
                        }
                    }
                    
                    // 如果是阿里云TV系统，那么我们默认识别层android
                    if(_isYunOSTvSystem()){
                        lDeviceInfoMap.put(UTLogFieldsScheme.OS.toString(), "a");
                    }

                    try {
                        Configuration configuration = new Configuration();
                        Settings.System.getConfiguration(
                                pContext.getContentResolver(), configuration);
                        if ((configuration != null)
                                && (configuration.locale != null)) {
                            lDeviceInfoMap.put(UTLogFieldsScheme.LANGUAGE.toString(),
                                    configuration.locale.toString());
                        } else {
                            lDeviceInfoMap.put(UTLogFieldsScheme.LANGUAGE.toString(), "Unknown");
                        }

                        DisplayMetrics dm = pContext.getResources()
                                .getDisplayMetrics();
                        int width = dm.widthPixels;
                        int height = dm.heightPixels;
                        if (width > height) {
                            width ^= height;
                            height ^= width;
                            width ^= height;
                        }
                        lDeviceInfoMap.put(UTLogFieldsScheme.RESOLUTION.toString(), height + "*"
                                + width);
                    } catch (Exception e) {
                        lDeviceInfoMap.put(UTLogFieldsScheme.RESOLUTION.toString(), "Unknown");
                    }

                    try {
                        String[] networkStatus = UTMCNetworkUtils
                                .getNetworkState(pContext);
                        lDeviceInfoMap.put(UTLogFieldsScheme.ACCESS.toString(), networkStatus[0]);
                        if (networkStatus[0].equals("2G/3G"))
                            lDeviceInfoMap.put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(),
                                    networkStatus[1]);
                        else {
                            lDeviceInfoMap.put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(),
                                    "Unknown");
                        }
                    } catch (Exception e) {
                        lDeviceInfoMap.put(UTLogFieldsScheme.ACCESS.toString(), "Unknown");
                        lDeviceInfoMap
                                .put(UTLogFieldsScheme.ACCESS_SUBTYPE.toString(), "Unknown");
                    }

                    try {
                        TelephonyManager telephonyManager = (TelephonyManager) pContext
                                .getSystemService("phone");
                        String networkName = "";
                        if (telephonyManager != null) {
                            if (telephonyManager.getSimState() == TelephonyManager.SIM_STATE_READY) {
                                networkName = telephonyManager.getNetworkOperatorName();
                            }
                        }
                        if (UTMCStringUtils.isEmpty(networkName)) {
                            networkName = "Unknown";
                        }
                        lDeviceInfoMap.put(UTLogFieldsScheme.CARRIER.toString(), networkName);
                    } catch (Exception e) {
                    	e.printStackTrace();
                    }
                    
                    try{
                        // 采集Mac地址
                        int lPermission = pContext
                                .checkCallingOrSelfPermission(Manifest.permission.ACCESS_WIFI_STATE);
                        
                        // 权限判断
                        if (PackageManager.PERMISSION_GRANTED == lPermission) {
                            String lMacAddr = UTMCNetworkUtils.getWifiAddress(pContext);
                            if (!UTMCStringUtils.isEmpty(lMacAddr)) {
                            	lDeviceInfoMap.put("_mac", lMacAddr);

                            }
                        }

                    }catch(Exception e){
                    	
                    }

                } catch (Exception e) {
                    return null;
                }
            }
            s_deviceInfoMap = lDeviceInfoMap;
            return s_deviceInfoMap;
        }
        return null;
    }

    /**
     * 判断是否是阿里云系统(TV or Phone)
     * @return
     */
    private static boolean isYunOSSystem() {
    	if ((System.getProperty("java.vm.name") != null && System
                .getProperty("java.vm.name").toLowerCase().contains("lemur"))
                || (null != System.getProperty("ro.yunos.version"))
                || (!TextUtils.isEmpty(SystemProperties.get("ro.yunos.build.version"))) ) {
            return true;
    	} else {
        	// 如果不是手机的，那么我们看看是否可能是aliyun TV
        	return _isYunOSTvSystem();
//            return false;
        }
    }
    
    private static boolean _isYunOSTvSystem() {
    	 if (!UTMCStringUtils.isEmpty(getSystemProperties("ro.yunos.product.chip"))
                 || !UTMCStringUtils.isEmpty(getSystemProperties("ro.yunos.hardware"))){
             return true;
    	 }else{
             return false;
    	 }
    }
    
    public static String getSystemProperties(String key) {
        String value = null;
        Class<?> cls = null;
        try {
            cls = Class.forName("android.os.SystemProperties");
            Method hideMethod = cls.getMethod("get", String.class);
            Object object = cls.newInstance();
            value = (String) hideMethod.invoke(object, key);
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return value;
    }

    /**
     * 获取UUID（TV or phone）
     * @return
     */
    private static String getUUID() {
        String lUUID = null;
        //for yunos
        lUUID = SystemProperties.get("ro.aliyun.clouduuid", "false");
        if("false".equals(lUUID)) {
            //for meizu
        	lUUID = SystemProperties.get("ro.sys.aliyun.clouduuid", "false");
        }        
        // 如果获取到UUID为空，那么我们尝试获取TV的UUID
        if (UTMCStringUtils.isEmpty(lUUID)){
        	return _getYunOSTVUuid();
        }
        
        return lUUID;
    }

    private static String _getYunOSTVUuid() {
    	String lUUID = null;
        try {
            Class<?> cloudUuid = Class.forName("com.yunos.baseservice.clouduuid.CloudUUID");
            Method m = cloudUuid.getMethod("getCloudUUID");
            lUUID = (String) m.invoke(null);
        } catch (Exception e) {
        }
        return lUUID;
    }
    
    private static String _getBuildVersion() {

        Field lField;
        try {
            lField = Build.class.getDeclaredField("YUNOS_BUILD_VERSION");
            if (null != lField) {
                lField.setAccessible(true);
                String fieldValue = (String) lField.get(new String());
                return fieldValue;
            }
        } catch (NoSuchFieldException e) {
//            e.printStackTrace();
        } catch (IllegalArgumentException e) {
//            e.printStackTrace();
        } catch (IllegalAccessException e) {
//            e.printStackTrace();
        }
        return null;
    }

}
