package com.ut.mini.core;

import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import android.app.Activity;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.appstatus.UTMCAppBackgroundTimeoutDetector;
import com.ut.mini.core.appstatus.UTMCAppStatusCallbacks;
import com.ut.mini.core.appstatus.UTMCAppStatusRegHelper;
import com.ut.mini.core.esg.UTMCEventStreamGroupBiz;
import com.ut.mini.core.logcache.UTMCGetCacheLogHelper;
import com.ut.mini.core.logcache.UTMCLogCacheHelper;
import com.ut.mini.core.loghelper.UTMCLogAssemble;
import com.ut.mini.core.request.UTMCTransferDataBuilder;
import com.ut.mini.core.request.UTMCUrlWrapper;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.module.UTMCScheduleMgr;
import com.ut.mini.plugin.UTMsgIds;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.utils.LogUtil;
import com.ut.mini.utils.SpSetting;
import com.ut.mini.utils.UTMCApiResponse;
import com.ut.mini.utils.UTMCHttpUtils;
import com.ut.mini.utils.UTMCNetworkUtils;
import com.ut.mini.utils.UTMCStringUtils;
import com.ut.store.UTLogStoreMgr;

public class UTMCLogTransfer implements UTMCAppStatusCallbacks {

	private static UTMCLogTransfer	s_instance	= new UTMCLogTransfer();

	private UTMCLogTransfer() {
	}

	public static UTMCLogTransfer getInstance() {
		return s_instance;
	}

	private volatile UTMCLogCacheHelper mLogCacheHelper = null;
	public void setmLogCacheHelper(UTMCLogCacheHelper mLogCacheHelper) {
		this.mLogCacheHelper = mLogCacheHelper;
	}

	private ScheduledFuture<?> mFixedTransferScheduledFuture = null;
	private boolean mIsTransferLogThreadRunning = false;
	private int mCurPackRecordCount = 20;
	private volatile UTMCGetCacheLogHelper mGetCacheLogHelper = null;
	private static final long		TWENTY_MIN					= 20 * 60 * 1000;
	private static final String		MUTI_PROCESS_UPLOAD_LOCK	= "upload_lock";
	/**
	 * scheduleAtFixedRate这个方法调度有问题，取消一个
	 * scheduleAtFixedRate，因为可以设定5s之后调度一次，再接着30s
	 */
	public void initialize() {

		// 启动5s后执行一次上传
		UTMCScheduleMgr.getInstance().schedule(new UTMCLogTransferRunnable(), 5, TimeUnit.SECONDS);

		long interval = Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH ? 600 * 1000 : 30 * 1000;

		mFixedTransferScheduledFuture = UTMCScheduleMgr.getInstance().scheduleWithFixedDelay(new UTMCLogTransferRunnable(), interval, interval,
				TimeUnit.MILLISECONDS);

		// Fix:如果版本是4.0 以上，注册app状态监测
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
			UTMCAppStatusRegHelper.registerAppStatusCallbacks(this);
			UTMCAppStatusRegHelper.registerAppStatusCallbacks(UTMCAppBackgroundTimeoutDetector.getInstance());
		}

	}

	public void saveCacheDataToLocal() {
		if (this.mLogCacheHelper != null) {
			this.mLogCacheHelper.saveToStorage();
		}
	}

	public void dispatchLocalHits() {
		_sendLog();
	}

	/**
	 * 派发日志
	 * 
	 * @param pEventID
	 *            行为ID
	 * @param aLogContent
	 *            行为日志 会优先存储，再check上传策略，如果满足就走上传逻辑
	 */
	public void dispatch(Map<String,String> logMap) {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG,"usertrack", "[dispatch] eventId:"+(logMap==null? "null":logMap.get(UTLogFieldsScheme.EVENTID.toString())));
		if (null == mLogCacheHelper) {
			mLogCacheHelper = UTMCLogCacheHelper.initWithContext(UTMCStatConfig.getInstance().getContext());

			// 初始化获取缓存的helper，并且绑定策略到达的
			mGetCacheLogHelper = new UTMCGetCacheLogHelper(mLogCacheHelper);
			UTMCEventStreamGroupBiz.getInstance().addStrategyArrivedListener(mGetCacheLogHelper);
		}
		
		if (logMap == null) {
			return ;
		}
		String eventId  = logMap.get(UTLogFieldsScheme.EVENTID.toString());
		List<String> lStreamNames = UTMCEventStreamGroupBiz.getInstance().calStreamNames(logMap);
		
		// 如果是drop的，那么我们需要把这个日志直接丢弃，要不然会影响我们的缓存。导致有效的数据缓存不进去。
		if (null != lStreamNames && lStreamNames.contains("drop")) {
				// 直接返回
			UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, "usertrack","direct drop" + " eventId:" +eventId);
			if (UTMCLogger.isDev()) {
				if (null != logMap) {
					UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "(" + logMap.get(UTLogFieldsScheme.EVENTID.toString()) + ") is skipped by EventStrategier:", logMap);
				}
			}
			return;
		}

		String index = LogUtil.generateLogIndex();
		String pPriority =null;
		if (logMap.containsKey(UTMCConstants.LogContentKeys.PRIORITY)) {
			pPriority = UTMCStringUtils.convertObjectToString(logMap.get(UTMCConstants.LogContentKeys.PRIORITY));
		}
		
		logMap.put(UTLogFieldsScheme.RESERVE3.toString(), index);
		String aLogContent = UTMCLogAssemble.assemble(logMap); ;
		if (!UTMCStringUtils.isEmpty(aLogContent)) {
			if (aLogContent.length() > UTMCConstants.G_MAX_PER_LOG_SIZE) {
				UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "dispatch log error", "log size is too long, log = " + aLogContent);
				return;
			}
		}
		if (null != mLogCacheHelper) {
			String lCacheKey = mLogCacheHelper.cacheLog_sqlite(eventId,pPriority,index, aLogContent);
			// 比配crash条件有问题
			if ("1".equalsIgnoreCase(eventId)) {
				mLogCacheHelper.saveToStorage();
				UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "crashhandler", "save crash log");
			}

			// 如果是实时调试，那么我们再发送一下数据
			if (UTMCVariables.getInstance().isRealTimeDebug()) {
				_sendLog();
			}
			// UTMCABTest.getInstance().abtest(aLogContent);

			/**
			 * Date:2015年 4月27日 星期一 17时08分04秒 CST 戴码
			 * 如果是测试模式，那么我们把如缓存的日志消息告诉外部的测试脚本
			 */
			if (UTMCStatConfig.getInstance().getBuildInfo().isTestMode()) {
				Map<String, String> lMap = UTMCLogAssemble.disassemble(aLogContent);
				if (null != lMap) {
					lMap.put("LOG_PRIORITY", pPriority);
					lMap.put("LOG_CACHE_KEY", lCacheKey);
				}
				UTPluginMgr.getInstance().dispatchPluginMsg(UTMsgIds.IN_LOG_CACHED_4_TEST_MODE, lMap);
			}

		}
	}
	
	/**
	 * 戴码添加：更新数据上传策略（start）
	 */
	protected static final String	TAG							= "UploadTask";

	private static final int		Default_WIN_SIZE			= 4;
	private static final int		MAX_LOG_COUNT				= 350;

	int								mWinSize					= -1;
	static int						mFactor						= 0;
	
	private int _getLogCount() {
		if (mWinSize == -1) {
			// 初试值
			String type = UTMCNetworkUtils.getNetworkType(UTMCStatConfig.getInstance().getContext());
			if (UTMCNetworkUtils.NETWORK_CLASS_WIFI.equalsIgnoreCase(type)) {
				mWinSize = Default_WIN_SIZE * 5;
			} else if (UTMCNetworkUtils.NETWORK_CLASS_4_G.equalsIgnoreCase(type)) {
				mWinSize = Default_WIN_SIZE * 4;
			} else if (UTMCNetworkUtils.NETWORK_CLASS_3_G.equalsIgnoreCase(type)) {
				mWinSize = Default_WIN_SIZE * 3;
			} else {
				mWinSize = Default_WIN_SIZE * 2;
			}
		}
		return mWinSize;
	}
	
	private int _calPackPackageWindowSize(Boolean isSuccess, long pTransferInterval,int aUploadPackageSize,float aAvgPackageSize) {

		if (pTransferInterval < 0) {
			return mWinSize;
		}
		int size = aUploadPackageSize;
		float currentSpeed = (float) size / pTransferInterval;

		if (isSuccess) {
			if (pTransferInterval > UTMCHttpUtils.MAX_READ_CONNECTION_STREAM_TIME_OUT - 15 * 1000) {
				return mWinSize;
			} else {
				double nextExpectByte = (UTMCHttpUtils.MAX_READ_CONNECTION_STREAM_TIME_OUT - 15 * 1000) * currentSpeed;
				mWinSize = (int) (nextExpectByte / aAvgPackageSize - mFactor);
			}
		} else {
			mWinSize = mWinSize / 2;
			mFactor++;
		}
		if (mWinSize < 1) {
			mWinSize = 1;
			mFactor = 0;
		} else if (mWinSize > MAX_LOG_COUNT) {
			mWinSize = MAX_LOG_COUNT;
		}
		UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, TAG, "winsize:" + mWinSize);
		return mWinSize;
	}
	/**
	 * 戴码添加：更新数据上传策略（finish）
	 */

	private class UTMCTransferLogRunnable implements Runnable {

		private boolean _reqServer(String aUrl, Map<String, Object> aBReqData) {
			if (null != aUrl) {
				byte[] lBResult = UTMCHttpUtils.sendRequest(UTMCHttpUtils.HTTP_REQ_TYPE_POST_FORM_DATA, aUrl, aBReqData, false);
				UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "request", aUrl);
				if (null != lBResult) {
					String lResult = null;
					try {
						lResult = new String(lBResult, "UTF-8");
					} catch (UnsupportedEncodingException e) {
						e.printStackTrace();
					}
					if (null != lResult) {
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "result", lResult);
						return UTMCApiResponse.parseResult(lResult);
					}

				}
			}
			return false;
		}

		public void run() {

			synchronized (UTMCLogTransfer.getInstance()) {
				if (mIsTransferLogThreadRunning) {
					return;
				} else {
					mIsTransferLogThreadRunning = true;
				}
			}
			try {
				//UTLogStoreMgr.getInstance().store();
				// 每次调度，最多上传3次，多了少了，我们都不再传送
				for (int lTransferCount = 0; lTransferCount < 3; lTransferCount++) {

					long time = SpSetting.get(UTMCStatConfig.getInstance().getContext(), MUTI_PROCESS_UPLOAD_LOCK);
					long interval = System.currentTimeMillis() - time;
					if (time == -1 || (interval > TWENTY_MIN)) {
						SpSetting.put(UTMCStatConfig.getInstance().getContext(), MUTI_PROCESS_UPLOAD_LOCK, System.currentTimeMillis());
					} else {
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV,"_sendLog", "Other Process is Uploading, break");
						break;
					}
					// before we upload, save SP changes to memory at least,
					// otherwise, changes will not be seen until apply/commit
					// called
					UTLogStoreMgr.getInstance().store();
					boolean lIsCEC = UTMCEventStreamGroupBiz.getInstance().isCEC();

					UTMCGetCacheLogHelper.GetCacheLogResult lGCLR = null;

					// 如果是实时调试，那么我们直接发送一批数据(日志时间从旧到新)
					Log.i("1010","get in transf");
					if (UTMCVariables.getInstance().isRealTimeDebug()) {
						
						lGCLR = mGetCacheLogHelper.getCacheLog_sqlite(_getLogCount(), lIsCEC, true, null);
					} else {
						/* 戴码修改：在没有获取到配置的时候，不走白名单，全部数据都上传
						// 日志排序规则（从新到旧）
						if (UTMCVariables.getInstance().isSyncOnlineConfSuccess()) {
							lGCLR = mGetCacheLogHelper.getCacheLog_sqlite(_getLogCount(), lIsCEC, false, null);
						} else {
							// 如果线上配置还未获取到，那么我们提取2001，1010等来做为上传条件
							UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "_sendLog", "skip[isSyncOnlineConfSuccess=false]");
							lGCLR = mGetCacheLogHelper.getCacheLog_sqlite(_getLogCount(), lIsCEC, false, sUrgentEventList);
						}*/
						lGCLR = mGetCacheLogHelper.getCacheLog_sqlite(_getLogCount(), lIsCEC, false, null);
					}

					if (null == lGCLR || lGCLR.getCacheLogCount() == 0) {
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "request[transfer_data]", "skip[no logs]");
						break;

					}
					if (null != lGCLR && lGCLR.getCacheLogCount() > 0) {

						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "reqeust[transfer_data]", "mCurPackRecordCount=" + mCurPackRecordCount);

//						boolean mIsSGStrategier = UTMCEventStreamGroupBiz.getInstance().isStrategierOK();

						Map<String, Object> lBReqData = UTMCTransferDataBuilder.buildPostRequestMap(lGCLR);

						// 如果有数据可以被发送
						if (null != lBReqData && lBReqData.size() > 0) {
							try {
								long lStart = SystemClock.elapsedRealtime();

								String lTransferUrl = UTMCUrlWrapper.getSignedTransferUrl(UTMCConstants.getTransferUrl(), null, lBReqData);

								boolean lIsSendSuccess = false;
								// 清理日志
								if (_reqServer(lTransferUrl, lBReqData)) {
									lIsSendSuccess = true;
									if (lGCLR.isHas2001Event()) {
										UTMCStatConfig.getInstance().setIsOldDevice(true);
									}
									mLogCacheHelper.clearCacheLog_sqlite(lGCLR.getUTLogs());
									if (lGCLR.getUTLogs()!=null) {
										UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV,"upload log","size:"+lGCLR.getUTLogs().size());
									}
								}
								long lDuration = SystemClock.elapsedRealtime() - lStart;
								UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "request[transfer-data]", "delay＝" + lDuration + "ms");
								// 调整下一次组包的大小
								_calPackPackageWindowSize(lIsSendSuccess,lDuration,lGCLR.getUploadPackageSize(),lGCLR.getPackageAvgSize());
							} catch (Exception e) {
								e.printStackTrace();
							}
						}
						// 再控制网络速率
					}
					SpSetting.put(UTMCStatConfig.getInstance().getContext(), MUTI_PROCESS_UPLOAD_LOCK, -1);
				}
			} catch (Throwable e) {
				e.printStackTrace();
			} finally {
				// 戴码：20150329 下午 5点
				// 严重程度：手机淘宝1200个crash中，会占到200多个，还是相当的频繁。
				// mLogCacheHelper空指针(目前发现的一个case：UTMCLogTransferMain.getInstance().transferLog(null)会导致这个bug)
				// 但是这个类的这个方法虽然是public的，但是外部没调用过（后期和小伟说的，尽量避免权限暴露的问题，在java中找一种解决方案解决包权限的控制）。
				// 应该还存在一个场景没找到。
				// TODO
				// 对这个case的修复代码应该需要继续深入调查。先用判空来处理一下（经过我的排查，我还是相信和初始化代码有关，如果是基于初始化的因素，
				// 在用户有打点的情况下，这个修复是不会影响数据质量的[这个case在启动那几秒的第一次上传会不成功【crash退出】，但是在后续的上传应该是可以生效的。]）
				if (null != mLogCacheHelper) {
					mLogCacheHelper.saveToStorage();
				}
				mIsTransferLogThreadRunning = false;
				SpSetting.put(UTMCStatConfig.getInstance().getContext(), MUTI_PROCESS_UPLOAD_LOCK, -1);
			}
		}
	}

	private class UTMCLogTransferRunnable implements Runnable {

		@Override
		public void run() {
			_sendLog();
		}

	}

	/**
	 * 发送日志调度。
	 */
	private synchronized void _sendLog() {
		UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "_sendLog", "_sendlog");

		// TODO 看是否需要删除这个判断代码
		boolean lIsConnectedInternet = UTMCNetworkUtils.isConnectInternet(UTMCStatConfig.getInstance().getContext());

		if (!lIsConnectedInternet) {
			UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "_sendLog", "skip[No ActiveNetworkInfo]");
			return;
		}

		if (mIsTransferLogThreadRunning) {
			UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "_sendLog", "mIsTransferLogThreadRunning=true");
			return;
		}

		UTMCScheduleMgr.getInstance().submit(new UTMCTransferLogRunnable());

	}

	/******************************** 应用活动监测 **********************************/
	private Object	mFixedTransferRateAdjLockObj	= new Object();

	private void _adjTransferRate(int aMillSeconds) {
		synchronized (mFixedTransferRateAdjLockObj) {

			if (null != mFixedTransferScheduledFuture) {
				mFixedTransferScheduledFuture.cancel(true);
			}

			mFixedTransferScheduledFuture = UTMCScheduleMgr.getInstance().scheduleWithFixedDelay(new UTMCLogTransferRunnable(), aMillSeconds, aMillSeconds,
					TimeUnit.MILLISECONDS);
			UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "_adjTransferRate", "millseconds:" + aMillSeconds);
		}
	}

	@Override
	public void onSwitchBackground() {
		UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "_sendlog", "in background");
		_sendLog();
		// 调整到 10 分钟调度一次：这样可以更加节能
		_adjTransferRate(600 * 1000);
		// when we switch to background, save it in case we never come back...
		if (null != mLogCacheHelper) {
			
			mLogCacheHelper.saveToStorage();
		}
	}

	@Override
	public void onSwitchForeground() {
		_adjTransferRate(30 * 1000);
	}

	@Override
	public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

	}

	@Override
	public void onActivityDestroyed(Activity activity) {

	}

	@Override
	public void onActivityPaused(Activity activity) {

	}

	@Override
	public void onActivityResumed(Activity activity) {

	}

	@Override
	public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

	}

	@Override
	public void onActivityStarted(Activity activity) {

	}

	@Override
	public void onActivityStopped(Activity activity) {

	}

}