package com.ut.mini.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTMCLogFields;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.esg.UTMCEventStreamGroupBiz;
import com.ut.mini.core.esg.strategy.UTMCSimpleEventIDStrategier;
import com.ut.mini.core.esg.strategy.UTMCSimpleEventIDStrategier.UTMCStrategyResult;
import com.ut.mini.core.loghelper.UTMCLogAssemble;
import com.ut.mini.core.onlineconf.UTMCRealTimeDebuggingBiz;
import com.ut.mini.core.onlineconf.UTMCTPKBiz;
import com.ut.mini.core.onlineconf.core.UTMCOnlineConfBiz;
import com.ut.mini.core.onlineconf.core.UTMCOnlineConfMgr;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.module.UTNetWorkStatusChecker;
import com.ut.mini.plugin.UTMsgIds;
import com.ut.mini.plugin.UTPlugin;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.plugin.UTPluginMsgDispatchDelegate;
import com.ut.mini.plugin.imps.UTAggregatedLogPlugin;
import com.ut.mini.utils.UTMCReflectUtils;
import com.ut.mini.utils.LogUtil;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * 数据传输入口类
 * 
 * @author daichenjie 以及做统一的资源控制
 */
public class UTMCLogTransferMain extends UTMCOnlineConfBiz implements UTMCSendLogDelegate.ISendLogListener {

	
	private final static String ONLINE_CONF_B01N19 = "B01N19";//这个配置，告诉数据由性能SDK来处理
	private boolean mIsDirectReturnWhenB01n19Match = false;//TODO 正式版本需要修改为true
	private UTMCSimpleEventIDStrategier mWhiteListStrategier = null;
	private Object mAccessWhiteListStrategierLockObj = new Object();
	private static UTMCLogTransferMain s_instance = new UTMCLogTransferMain();
	private volatile boolean mIsInitialized = false; // 要让大家都能立马看得到
	private UTMCSendLogDelegate mSendLogDelegate = null;
	private Object mInitializeLockObj = new Object();

	private UTMCLogTransferMain() {

	}

	public static UTMCLogTransferMain getInstance() {
		return s_instance;
	}

	private void _initialize() {
		if(!mIsInitialized){
			synchronized(mInitializeLockObj){
				if (null != UTMCStatConfig.getInstance().getContext()) {
					
					// 初始化Transfer的,这里很多钩子需要同步初始化才可以
					UTMCLogTransfer.getInstance().initialize();
					
					// 绑定聚合数据接收插件
					UTPluginMgr.getInstance().registerPlugin(new UTAggregatedLogPlugin(), false);
					// 启动合作方插件
					UTPluginMgr.getInstance().runPartnerPlugin();
					
					UTMCOnlineConfMgr lOnlineConfMgr = new UTMCOnlineConfMgr();
					// 日志分组配置
					lOnlineConfMgr.addConfBiz(UTMCEventStreamGroupBiz.getInstance());
					lOnlineConfMgr.addConfBiz(UTMCRealTimeDebuggingBiz.getInstance());
					lOnlineConfMgr.addConfBiz(UTMCTPKBiz.getInstance());
					lOnlineConfMgr.addConfBiz(this);
					lOnlineConfMgr.syncOnlineConfs();
					mSendLogDelegate = new UTMCSendLogDelegate();
					mSendLogDelegate.setSendLogListener(this);
					mSendLogDelegate.start();
					
					UTNetWorkStatusChecker.enable(UTMCStatConfig.getInstance().getContext());
					
					mIsInitialized = true;
				}
			}
		}
	}

	/**
	 * 日志输出的总入口
	 * 
	 * @param aLogMap
	 *            包含log内容的map对象
	 */
	public void transferLog(Map<String, String> aLogMap) {
		// 这里不用做同步保护，没有关系
		if (!mIsInitialized) {
			// lazy初始化
			_initialize();
		}
		
		// 如果发送的log为空，那么我们仅仅是激活当前的通道
		if (null == aLogMap) {
			if(null != mSendLogDelegate){
				mSendLogDelegate.send(null);
			}
			return;
		}

		// context 和appkey必须都不能为空
		if (null == UTMCStatConfig.getInstance().getContext()
				|| UTMCStringUtils.isEmpty(UTMCStatConfig.getInstance()
						.getAppkey())) {
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "setRequestAuthentication", "Fatal Error,no appkey was setted in RequestAuthentication");
			return;
		}
		
		if(null != mSendLogDelegate){
			mSendLogDelegate.send(aLogMap);
		}
		
	}

	private void _transferLog(Map<String, String> aLogMap) {
		if (null != aLogMap) {

			// send dispatch log notification
			Map<String,String> lDispatchLogMap = new HashMap<String,String>();
			lDispatchLogMap.putAll(aLogMap);
			UTPluginMgr.getInstance().dispatchPluginMsg(UTMsgIds.IN_DISPATCH_SIMPLE_LOG_MAP, new UTPluginMsgDispatchDelegate(lDispatchLogMap){

				@Override
				public Object getDispatchObject(UTPlugin lPlugin) {
					Map<String , String> lLogMap = (Map<String, String>)this.getMsgObj();
					Map<String , String> lNewLogMap = new HashMap<String,String>();
					lNewLogMap.putAll(lLogMap);
					return lNewLogMap;
				}
			});

			// 读取日志的优先级
			boolean lIsAggregationLog = false;
			if(aLogMap.containsKey(UTMCLogFields.AGGREGATION_LOG.toString())){
				lIsAggregationLog = true;
				aLogMap.remove(UTMCLogFields.AGGREGATION_LOG.toString());
			}
			String lEventID = aLogMap.get(UTLogFieldsScheme.EVENTID.toString());
			// 如果是聚合事件，那么我们把聚合的事件派发给插件处理
			if(lIsAggregationLog){
				try{
					Map<String, String> lNewLogMap = new HashMap<String,String>();
					lNewLogMap.putAll(aLogMap);
					Map<String, String> lNewLogMap2 = new HashMap<String,String>();
					lNewLogMap2.putAll(aLogMap);
					String lLogContent = UTMCLogAssemble.assemble(lNewLogMap2);
					Map<String,String> lNewLogMap3 = UTMCLogAssemble.disassemble(lLogContent);
					lNewLogMap.putAll(lNewLogMap3);
					lNewLogMap.remove(UTLogFieldsScheme.ARGS.toString());
					UTPluginMgr.getInstance().dispatchPluginMsg(UTMsgIds.IN_DISPATCH_AGGREGATION_LOG_MAP, lNewLogMap);
				}catch(Exception e){
					e.printStackTrace();
				}
			}else{// 如果非聚合事件，那么我们把事件派发给内部SDK去消化处理.
				
//				aLogMap.put(UTLogFieldsScheme.RESERVE3.toString(), index);
//				String lLogContent = UTMCLogAssemble.assemble(aLogMap);
				
				// 如果拼装好的日志不为空
//				if (!UTMCStringUtils.isEmpty(lLogContent)) {
//					if(UTPluginMgr.getInstance().isPartnerPluginExist(UTPluginMgr.PARTNERPLUGIN_UTPREF)){
//						// 如果合作的性能数据采集SDK存在
//						UTMCStrategyResult lSResult = null;
//						int lIntEventID = 0;
//						synchronized(mAccessWhiteListStrategierLockObj){
//							if(null != mWhiteListStrategier){
//								lIntEventID = 0;
//								try{
//									lIntEventID = Integer.parseInt(lEventID);
//								}catch(Exception e){
//									e.printStackTrace();
//								}
//								if(0 != lIntEventID){
//									lSResult = mWhiteListStrategier.isOK(lIntEventID);
//								}
//							}
//						}
//						
//						if(null != lSResult && lSResult.isExist()){
//							
//							// 白名单中存在这个EventID
//							if(lSResult.isOk()){
//								if(UTMCLogger.isDebug()){
//									UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "LogTransfer", "send to utperf success("+lIntEventID+").");
//								}
//								// 如果采样率OK
//								UTPluginMgr.getInstance().dispatchPluginMsg(
//										UTPlugin.MSG_ID_IN_DISPATCH_LOG_STRING_4_UTPERF_PLUGIN, lLogContent);
//							}else{
//								if(UTMCLogger.isDebug()){
//									UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "LogTransfer", "send to utpref sdk fail("+lIntEventID+":Sampling failure).");
//								}
//							}
//							
//							if(mIsDirectReturnWhenB01n19Match){
//								// 正式版本，这个流量直接给性能SDK消费，UT 中不重复打点
//								return;
//							}
//						}
//					}
					// 派发日志
					UTMCLogTransfer.getInstance().dispatch(aLogMap);
//				}
			}
			
			// 如果是聚合模式，以及是实时调试情况下
			// 修改eventID 字段为 6699，并且把老的eventID放到args中去。
			if(lIsAggregationLog){
				if(UTMCVariables.getInstance().isRealTimeDebug()){
					Map<String, String> lNewLogMap = new HashMap<String,String>();
					lNewLogMap.putAll(aLogMap);
					if(!UTMCStringUtils.isEmpty(lEventID)){
						lNewLogMap.remove(UTLogFieldsScheme.EVENTID.toString());
						lNewLogMap.put(UTLogFieldsScheme.EVENTID.toString(), ""+UTMCConstants.EventID.AGGREGATION_LOG);
						lNewLogMap.put("_event_id", lEventID);
						// 发送log出去。
//						String lLogContent = UTMCLogAssemble.assemble(lNewLogMap);
						// 如果拼装好的日志不为空
//						if (!UTMCStringUtils.isEmpty(lLogContent)) {
//							// 派发日志
							UTMCLogTransfer.getInstance().dispatch(lNewLogMap);
//						}
						
					}
				}
			}
			
		}
	}
	
	@Override
	public void onLogArrived(Map<String, String> aLogMap) {
		if(null == aLogMap){
			UTMCLogTransfer.getInstance().dispatch(null);
		}else{
			_transferLog(aLogMap);
		}
	}

	@Override
	public void onConfigurationArrive(String pConfName, String pConfContent) {
		if(pConfContent instanceof String){
	        //whitelist
	        UTMCSimpleEventIDStrategier lSimapleEventIDStrategier = new UTMCSimpleEventIDStrategier();
	        try {
	            JSONArray lJsonArray = new JSONArray(pConfContent);
	                for (int i = 0; i < lJsonArray.length(); i++) {
	                    JSONObject jData = null;
	                    try {
	                        jData = lJsonArray.getJSONObject(i);
	                    } catch (JSONException e) {
	                    }
	 
	                    if (jData != null && jData.has("e")) {
	                    try {
	                        int eventId = jData.getInt("e");
	                        int cp = 100;
	                        if(jData.has("cp")){
	                        	cp = jData.getInt("cp");
	                        }
	                        if (cp >= 0 && cp <= 100) {
	                            lSimapleEventIDStrategier.addEventIDStrategy(eventId, cp);
	                        }
	                    } catch (JSONException e) {
	                    }
	                }
	            }
	            synchronized(mAccessWhiteListStrategierLockObj){
	            	mWhiteListStrategier = lSimapleEventIDStrategier;
	            }
	        } catch (JSONException e1) {
	            e1.printStackTrace();
	        }
		}
	}

	@Override
	public List<String> returnRequiredConfigurationNameList() {
		List<String> lRConfNames = new ArrayList<String>();
		lRConfNames.add(ONLINE_CONF_B01N19);
		return lRConfNames;
	}
	
}
