package com.ut.mini.core;

import java.util.Map;

import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCReflectUtils;

import android.content.Context;
import android.content.SharedPreferences;

public class UTMCVariables {
	
	public final static UTMCVariables s_instance = new UTMCVariables();
	
	public static UTMCVariables getInstance(){
		return s_instance;
	}
	
	private boolean mIsRealTimeDebugging = false;
    private boolean mSyncOnlineConfStatus = false;
    private String mDebuggingKey = null;
    private Map<String,String> mSessionProperties = null;
    private boolean mDebugSamplingOption = false;
    private boolean mIs1010AutoTrackClosed = false;
    // 这个TTID，要放到tpk＝{ttid=}这样的格式
    private String mOutsideTTID = null;
    
    public synchronized void setOutsideTTID(String aTTID){
    	mOutsideTTID = aTTID;
    }
    
    public synchronized String getOutsideTTID(){
    	return mOutsideTTID;
    }
    
    public synchronized void set1010AutoTrackClose(){
    	mIs1010AutoTrackClosed = true;
    }
    
    public synchronized boolean is1010AutoTrackClosed(){
    	return mIs1010AutoTrackClosed;
    }
    
    private String mHostPackageImei = null;//主包IMEI
    private String mHostPackageImsi = null;//主包IMEI
    private boolean mIsHostPackageImeiAndImsiRead = false;
    
    private synchronized void _setHostPackageImei(String aImei){
    	mHostPackageImei = aImei;
    }
    
    private synchronized void _setHostPackageImsi(String aImsi){
    	mHostPackageImsi = aImsi;
    }
    
    private synchronized void _getHostPackageImeiAndImsi(){
    	// 只读一次
    	if(mIsHostPackageImeiAndImsiRead){
    		return;
    	}
    	//初始化IMEI,IMSI,及时异常也只读一次
		try {
			Class<?> lClass = Class.forName("com.taobao.statistic.comp.imeisi.Device");
			if(null != lClass){
				Object[] lObj = new Object[1];
				lObj[0]=UTMCStatConfig.getInstance().getContext();
				Object lResultObj = UTMCReflectUtils.invokeStaticMethod(lClass, "getDevice",lObj,Context.class);
				if(null != lResultObj){
					Object lImeiObj = UTMCReflectUtils.invokeMethod(lResultObj, "getImei");
					Object lImsiObj = UTMCReflectUtils.invokeMethod(lResultObj, "getImsi");
					if(lImeiObj instanceof String && lImsiObj instanceof String){
						_setHostPackageImei((String)lImeiObj);
						_setHostPackageImsi((String)lImsiObj);
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "UT", "Read IMEI/IMSI from TBS.");
					}
				}
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		mIsHostPackageImeiAndImsiRead = true;
    }
    
    public synchronized String getHostPackageImei(){
    	_getHostPackageImeiAndImsi();
    	return mHostPackageImei;
    }
    
    public synchronized String getHostPackageImsi(){
    	_getHostPackageImeiAndImsi();
    	return mHostPackageImsi;
    }
    
    public synchronized void setDebugSamplingOption(){
    	mDebugSamplingOption = true;
    }
    
    public synchronized boolean getDebugSamplingOption(){
    	return mDebugSamplingOption;
    }
    
    public synchronized void setSessionProperties(Map<String,String> aMap){
    	mSessionProperties = aMap;
    }
    
    public synchronized Map<String,String> getSessionProperties(){
    	return mSessionProperties;
    }
    
    public synchronized void setDebugKey(String aDebuggingKey){
    	mDebuggingKey = aDebuggingKey;
    }
    
    public synchronized String getDebugKey(){
    	return mDebuggingKey;
    }

	public synchronized boolean isRealTimeDebug() {
		return mIsRealTimeDebugging;
	}

	public synchronized void setRealTimeDebugFlag() {
		mIsRealTimeDebugging = true;
	}
	
	public synchronized void resetRealTimeDebugFlag(){
		mIsRealTimeDebugging = false;
	}
	
	/**
     * 看是否同步线上的配置OK（如果有配置，会先通知配置业务模块）
     * @return
     */
    public synchronized boolean isSyncOnlineConfSuccess(){
    	return mSyncOnlineConfStatus;
    }
    
    public synchronized void setSyncOnlineConfStatus(boolean aStatus){
    	mSyncOnlineConfStatus = aStatus;
    }
    
}
