package com.ut.mini.core.esg;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;

import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.esg.strategy.UTMCEventStreamGroupStrategier;
import com.ut.mini.core.loghelper.UTMCLogAssemble;
import com.ut.mini.core.onlineconf.core.UTMCOnlineConfBiz;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCEventStreamGroupBiz extends UTMCOnlineConfBiz {

	private static UTMCEventStreamGroupBiz s_instance = null;
	private UTMCEventStreamGroupStrategier mEventStreamGroupStrategier = null;
	private boolean mIsCEC = false; // 19999 事件的格式兼容性
	private List<UTMCEventStreamGroupStrategyArrivedListener> mStrategierArrivedListeners = new LinkedList<UTMCEventStreamGroupStrategyArrivedListener>();

	private UTMCEventStreamGroupBiz() {

	}
	
	// 这个方法在编译的时候会被去除掉。
	// TODO 这个内容，android上有一个注解是能去对应的，这个我看到过，需要去寻找一下
	public synchronized void resetEventStreamGroupStrategierToNull(){
		mEventStreamGroupStrategier = null;
	}

	public synchronized static UTMCEventStreamGroupBiz getInstance() {
		if (null == s_instance) {
			s_instance = new UTMCEventStreamGroupBiz();
		}
		return s_instance;
	}

	public synchronized void addStrategyArrivedListener(
			UTMCEventStreamGroupStrategyArrivedListener aLinstener) {
		if (null != aLinstener) {
			mStrategierArrivedListeners.add(aLinstener);
		}
	}

	public synchronized void removeStrategyArrivedListener(
			UTMCEventStreamGroupStrategyArrivedListener aLinstener) {
		if (null != aLinstener) {
			mStrategierArrivedListeners.remove(aLinstener);
		}
	}

	private synchronized void _updateEventStreamGroupStrategier(
			UTMCEventStreamGroupStrategier aEventStreamGroupStrategier) {
		mEventStreamGroupStrategier = aEventStreamGroupStrategier;
		// 通知收到通知
		if (mStrategierArrivedListeners != null
				&& mStrategierArrivedListeners.size() > 0) {
			for (UTMCEventStreamGroupStrategyArrivedListener lListener : mStrategierArrivedListeners) {
				lListener.onEventStreamGroupStrategyArrived();
			}
		}
	}

	/**
	 * 读取配置到达状态
	 * 
	 * @return 如果到达，那么返回OK
	 */
	public synchronized boolean isStrategierOK() {
		if (null == mEventStreamGroupStrategier) {
			return false;
		}
		return true;
	}

	public boolean isCEC() {
		return mIsCEC;
	}

	public List<String> calStreamNames(String aLogContent) {
		if (null != aLogContent) {
			Map<String, String> lLogMap = UTMCLogAssemble
					.disassemble(aLogContent);
			return calStreamNames(lLogMap);
		}
		return null;
	}

	public synchronized List<String> calStreamNames(Map<String, String> aLogMap) {
		try {

			if(UTMCVariables.getInstance().isRealTimeDebug()){
				// 如果打开了调试平台打开的采样选项，那么我们所有的数据都不做采样，直接返回stm_d.
				if (UTMCVariables.getInstance().getDebugSamplingOption()){
					List<String> lStreamNames = new LinkedList<String>();
					lStreamNames.add("stm_d");
					return lStreamNames;
				}
			}
			
			if (null == aLogMap) {
				return null;
			}

			// 如果存在流策略，那么我们就推送到策略中去，否则，我们就推送到默认流.
			// 在我们加了网络控制之后，如果启动的时候，没有网络，那么这个配置就可能会没有，那么怎么办呢
			// 网络配置同步 这一块去修正这个bug,在没有网络的时候，派发本地配置。
			// 那么这里还是有一个问题，如果之前没有成果过，及时派发本地的配置，那这个也是空，那么我们是否允许去绕过去。
			List<String> lStreamNames = null;
			if (null != mEventStreamGroupStrategier) {
				// 匹配数据分组策略，获得，需要对应的 分组名称，组装数据
				lStreamNames = mEventStreamGroupStrategier
						.getStreamNames(aLogMap);
				
			}
			int lEventID = 0;
			if (aLogMap.containsKey(UTLogFieldsScheme.EVENTID.toString())) {
				String lEventStr = aLogMap.get(UTLogFieldsScheme.EVENTID.toString());
				if (null != lEventStr) {
					try {
						lEventID = Integer.parseInt(lEventStr);
					} catch (Exception e) {
					}
				}
			}
			// 如果线上返回的流不存在，或者没匹配上，那么我们默认加一个stm_nc。
			if (lStreamNames == null || lStreamNames.size() == 0) {
				// 如果是页面和控件，那么我们指定一个固定的名称去处理
				lStreamNames = new ArrayList<String>();
				// 如果是crash，那么我们在没获取到配置的时候，指定成stm_x
				if (lEventID == 1 || lEventID == 61006) {
					lStreamNames.add("stm_x");
				} else if (lEventID > 1000 && lEventID < 2100) {
					// 没有获取到配置的时候，页面的我们指定成stm_p
					lStreamNames.add("stm_p");
				} else if (lEventID > 2100 && lEventID < 2200) {
					// 没有获取到配置的时候，控件的我们指定成stm_c
					lStreamNames.add("stm_c");
				} else if (lEventID == 6699){// 6699 聚合eventID，聚合的时候，我们会把实际的eventID和6699发生交替【实时调试模式下】
					lStreamNames.add("stm_d");
				} else if (lEventID == 19999){
					lStreamNames.add("stm_d");
				} else {
					// 没有获取到配置的时候，其它，我都指定成stm_nc
					lStreamNames.add("stm_nc");
				}
			}
			//TODO 临时代码 增加对65115处理
			if (lEventID ==65115) {
				if (lStreamNames!=null) {
					if (lStreamNames.contains("drop")) {
						lStreamNames.clear();
						lStreamNames.add("stm_d");
					}
				}
			}
			
			UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG,"usertrack","[calStreamNames]"+"eventId:"+(aLogMap==null?"null":aLogMap.get(UTLogFieldsScheme.EVENTID.toString())) + " lStreamNames:" + lStreamNames);
			return lStreamNames;
		} catch (Exception e) {

		}
		return null;
	}

	@Override
	public void onConfigurationArrive(String pConfName, String pConfContent) {
		if (!UTMCStringUtils.isEmpty(pConfContent)) {
			boolean lCEC = false;
			try {

				JSONObject lJsonObj = new JSONObject(pConfContent);
				if (lJsonObj.has("cec")) {
					if (lJsonObj.getInt("cec") == 1) {
						lCEC = true;
					} else {
						lCEC = false;
					}
				}

				mIsCEC = lCEC;

				if (lJsonObj.has("stms")) {
					Log.d("usertrack", "pConfName"+pConfName+" pConfContent:"+pConfContent);
					UTMCEventStreamGroupStrategier lStrategier = new UTMCEventStreamGroupStrategier();
					lStrategier.addRules(lJsonObj);
					_updateEventStreamGroupStrategier(lStrategier);
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public List<String> returnRequiredConfigurationNameList() {
		List<String> lNeedConfigurations = new ArrayList<String>();
		lNeedConfigurations
				.add(UTMCConstants.OnlineConfBusiness.EVENT_STRAEM_GROUP);
		// 增加项
		return lNeedConfigurations;
	}

	@Override
	public void onNonConfigurationArrive(String pConfName) {

	}

}