package com.ut.mini.core.esg.strategy;

import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.core.esg.strategy.UTMCSimpleEventIDStrategier.UTMCStrategyResult;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * {“stm”:"stm_p","rs":[{"ers":[{'e':'',s:1|0}],"p" : ["detail"],"a1":["%含有%"
 * ],"a2":["完全相等"],"a3":["内容"],"as":["内容"]}] }
 */
public class UTMCEventStreamGroupRule {

	private String[] _getStringArray(String pKey, JSONObject pJsonObj) {
		JSONArray lJSONA;
		try {
			lJSONA = pJsonObj.getJSONArray(pKey);
			if (null != lJSONA && lJSONA.length() > 0) {
				String[] lStringA = new String[lJSONA.length()];
				for (int i = 0; i < lJSONA.length(); i++) {
					lStringA[i] = lJSONA.getString(i);
				}
				return lStringA;
			}
		} catch (JSONException e) {
			// e.printStackTrace();
		}
		return null;
	}

	public UTMCEventStreamGroupRule(JSONObject lJsonObj) {
		mEventIdStrategier = new UTMCSimpleEventIDStrategier();
		try {
			JSONArray lEids = lJsonObj.getJSONArray("ers");
			for (int i = 0; i < lEids.length(); i++) {
				JSONObject lJObj = lEids.getJSONObject(i);
				int lEventID = lJObj.optInt("e");
				// 这里变更s为cp，值的范围为[0,100]
				int lS = lJObj.optInt("cp");
				if (0 != lEventID && (lS >= 0 && lS <= 100)) {
					// 控制采样率
					mEventIdStrategier.addEventIDStrategy(lEventID, lS);
				}
				// 如果cp不存在，那么直接添加100%
				if (!lJObj.has("cp")) {
					mEventIdStrategier.addEventIDStrategy(lEventID, 100);
				}
			}

			mPage = _getStringArray("p", lJsonObj);
			mArg1 = _getStringArray("a1", lJsonObj);
			mArg2 = _getStringArray("a2", lJsonObj);
			mArg3 = _getStringArray("a3", lJsonObj);
			mArgs = _getStringArray("as", lJsonObj);

		} catch (JSONException e) {
			e.printStackTrace();
		}
	}

	/*
	 * 匹配数组和内容，看是否匹配的上
	 */
	private boolean _matchStringA(String pContent, String[] pArray) {
		if (null == pArray || pArray.length == 0) {
			return true;
		}
		try {
			if (!UTMCStringUtils.isEmpty(pContent) && pArray != null
					&& pArray.length > 0) {
				for (String lStr : pArray) {
					if (lStr.startsWith("%") && lStr.endsWith("%")) {
						// 截取不包含%%的内容
						lStr = lStr.substring(1, lStr.length() - 2);
						if (pContent.contains(lStr)) {
							return true;
						}
					} else {
						if (pContent.equals(lStr)) {
							return true;
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public UTMCStrategierMatchResult isESGMatch(Map<String, String> pLogContent) {
		UTMCStrategyResult lSResult = null;
		try {
			if (null != pLogContent && null != mEventIdStrategier) {
				String lEventIDStr = pLogContent
						.get(UTLogFieldsScheme.EVENTID.toString());
				int lEventID = Integer.valueOf(lEventIDStr).intValue();
				if (null != mEventIdStrategier) {
					// 先判断eventID是否存在，如果存在，那么我们接下来匹配采样率
					lSResult = mEventIdStrategier.isOK(lEventID);
					if (lSResult.isExist()) {
						String lPage = pLogContent.get(UTLogFieldsScheme.PAGE
								.toString());
						String lA1 = pLogContent.get(UTLogFieldsScheme.ARG1
								.toString());
						String lA2 = pLogContent.get(UTLogFieldsScheme.ARG2
								.toString());
						String lA3 = pLogContent.get(UTLogFieldsScheme.ARG3
								.toString());
						String lAs = pLogContent.get(UTLogFieldsScheme.ARGS
								.toString());
						if (_matchStringA(lPage, mPage)
								&& _matchStringA(lA1, mArg1)
								&& _matchStringA(lA2, mArg2)
								&& _matchStringA(lA3, mArg3)
								&& _matchStringA(lAs, mArgs)) {
							UTMCStrategierMatchResult lResult = new UTMCStrategierMatchResult();
							// 如果采样成功，那么我也对外部抛出的success
							if (lSResult.getCPResult()) {
								lResult.setCPSuccess();
							}
							lResult.setCP(lSResult.getCP());
							lResult.setIsMatch(true);
							return lResult;
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		UTMCStrategierMatchResult lResult = new UTMCStrategierMatchResult();
		// 上述的分支可能会泄漏，因为可能matcher不上，但是这个时候，可能是 存在这个EventID配置项的。
		lResult.setIsMatch(false);
		return lResult;
	}

	private UTMCSimpleEventIDStrategier mEventIdStrategier = null;
	private String[] mPage = null;
	private String[] mArg1 = null;
	private String[] mArg2 = null;
	private String[] mArg3 = null;
	private String[] mArgs = null;
}