package com.ut.mini.core.esg.strategy;

import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class UTMCEventStreamGroupRuleGroup {

	private String mGroupName = null;
	private List<UTMCEventStreamGroupRule> lRules = new Vector<UTMCEventStreamGroupRule>();

	public String getGroupName() {
		return mGroupName;
	}

	/**
	 * {“stm”:"stm_p","rs":[{"ers":[{'e':'',s:'off'|'on'}],"p" :
	 * ["detail"],"a1":["%含有%"
	 * ],"a2":["完全相等"],"a3":["内容"],"as":["内容"]}],"cec":"" }
	 */
	public UTMCEventStreamGroupRuleGroup(String pGroupName, JSONArray pRules) {
		mGroupName = pGroupName;
		if (null != pRules) {
			for (int i = 0; i < pRules.length(); i++) {
				try {
					JSONObject lJObj = pRules.getJSONObject(i);
					if (lJObj.has("ers")) {
						lRules.add(new UTMCEventStreamGroupRule(lJObj));
					}
				} catch (JSONException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * 看日志是否满足匹配规则
	 * 
	 * @param pLogContent
	 *            日志内容
	 * @return 调用子匹配，如果匹配成功，那么我们直接返回，如果匹配失败，那么我们返回false.
	 */
	public UTMCStrategierMatchResult isESGMatch(Map<String, String> pLogContent) {

		for (UTMCEventStreamGroupRule lECG : lRules) {
			UTMCStrategierMatchResult lResult = lECG.isESGMatch(pLogContent);
			if (lResult.isMatch()) {
				return lResult;
			}
		}

		UTMCStrategierMatchResult lResult = new UTMCStrategierMatchResult();
		lResult.setIsMatch(false);
		return lResult;
	}

}