package com.ut.mini.core.esg.strategy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCEventStreamGroupStrategier {

	private Map<String, UTMCEventStreamGroupRuleGroup> mSpecECRGRuleGroups = new HashMap<String, UTMCEventStreamGroupRuleGroup>();
	private List<UTMCEventStreamGroupRuleGroup> mECGRRuleGroups = new Vector<UTMCEventStreamGroupRuleGroup>();

	public List<String> getStreamNames(Map<String, String> pLogContent) {
		List<UTMCEventStreamGroupRuleGroup> lResults = _getMatchRuleGroups(pLogContent);
		if (null == lResults) {
			return null;
		}
		// 遍历取出 合理的分组名，并且返回
		List<String> lRValues = new LinkedList<String>();
		for (UTMCEventStreamGroupRuleGroup lECGR : lResults) {
			if (!lRValues.contains(lECGR.getGroupName())) {
				lRValues.add(lECGR.getGroupName());
			}
		}
		return lRValues;
	}

	private List<UTMCEventStreamGroupRuleGroup> _getMatchRuleGroups(
			Map<String, String> pLogContent) {
		UTMCEventStreamGroupRuleGroup lECGR = null;
		if (mSpecECRGRuleGroups.containsKey("drop")) {
			lECGR = mSpecECRGRuleGroups.get("drop");
			UTMCStrategierMatchResult lMatchResult = lECGR
					.isESGMatch(pLogContent);
			if (lMatchResult.isMatch() && lMatchResult.getCPResult()) {
				// 日志直接drop
				List<UTMCEventStreamGroupRuleGroup> lResult = new ArrayList<UTMCEventStreamGroupRuleGroup>();
				lResult.add(lECGR);
				return lResult;
			}
		}

		if (mSpecECRGRuleGroups.containsKey("delay")) {
			lECGR = mSpecECRGRuleGroups.get("delay");
			UTMCStrategierMatchResult lMatchResult = lECGR
					.isESGMatch(pLogContent);
			if (lMatchResult.isMatch() && lMatchResult.getCPResult()) {
				List<UTMCEventStreamGroupRuleGroup> lResult = new ArrayList<UTMCEventStreamGroupRuleGroup>();
				lResult.add(lECGR);
				return lResult;
			}
		}
		
		List<UTMCEventStreamGroupRuleGroup> lResult = new ArrayList<UTMCEventStreamGroupRuleGroup>();
		// 遍历基本的 规则 模块
		boolean mIsAMatch = false;
		boolean mIsCPResult = true;
		for (UTMCEventStreamGroupRuleGroup lECGR2 : mECGRRuleGroups) {
			UTMCStrategierMatchResult lMatchResult = lECGR2.isESGMatch(pLogContent);
			if (lMatchResult.isMatch() && lMatchResult.getCPResult()) {
				lResult.add(lECGR2);
				mIsAMatch = true;
			} else {
				if (lMatchResult.isMatch() && !lMatchResult.getCPResult()) {
					mIsCPResult = false;
				}
			}
		}

		// 如果有match上的，并且数量大于0，那么我们直接返回
		if (mSpecECRGRuleGroups.containsKey("ki_stm_d")) {
			lECGR = mSpecECRGRuleGroups.get("ki_stm_d");
			UTMCStrategierMatchResult lMatchResult = lECGR
					.isESGMatch(pLogContent);
			if (lMatchResult.isMatch() && lMatchResult.getCPResult()) {
				// 把kidefault 重新导向到 default流。
				UTMCEventStreamGroupRuleGroup lECGRTmp = new UTMCEventStreamGroupRuleGroup(
						"stm_d", null);
				lResult.add(lECGRTmp);
				mIsAMatch = true;
			}else {
				if (lMatchResult.isMatch() && !lMatchResult.getCPResult()) {
					mIsCPResult = false;
				}
			}
		}

		// 如果没有match上的
		if (!mIsAMatch && !mIsCPResult) {
			UTMCEventStreamGroupRuleGroup lECGRTmp = new UTMCEventStreamGroupRuleGroup("drop", null);
			lResult.add(lECGRTmp);
			return lResult;
		}

		// 如果基本流匹配上，或者匹配上了kidefault流
		if (lResult.size() > 0) {
			return lResult;
		}

		// 如果配置了default流，那么我匹配一下，如果匹配上，那么就返回
		// 如果没有配置default流，那么我们返回一个default六
		if (mSpecECRGRuleGroups.containsKey("stm_d")) {
			lECGR = mSpecECRGRuleGroups.get("stm_d");
			UTMCStrategierMatchResult lMatchResult = lECGR
					.isESGMatch(pLogContent);
			if (lMatchResult.isMatch() && lMatchResult.getCPResult()) {
				lResult.add(lECGR);
				return lResult;
			} else {
				// 如果配置了stm_d，并且未有任何流匹配上的话，那么我们返回drop,外部直接抛弃
				UTMCEventStreamGroupRuleGroup lECGRTmp = new UTMCEventStreamGroupRuleGroup(
						"drop", null);
				lResult.add(lECGRTmp);
				return lResult;
			}
		} else {
			UTMCEventStreamGroupRuleGroup lECGRTmp = new UTMCEventStreamGroupRuleGroup(
					"stm_d", null);
			lResult.add(lECGRTmp);
			return lResult;
		}
	}

	public void addRules(JSONObject pJsonObj) {
		if (null != pJsonObj && pJsonObj.has("stms")) {
			JSONArray lJsonArray;
			try {
				lJsonArray = pJsonObj.getJSONArray("stms");
				for (int i = 0; i < lJsonArray.length(); i++) {
					JSONObject lJObj = lJsonArray.getJSONObject(i);
					if (null != lJObj) {
						_addRule(lJObj);
					}
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * {“stm”:"stm_p","rs":[{"eids":[2001,2002],"p" :
	 * ["detail"],"a1":["%含有%"],"a2":["完全相等"],"a3":["内容"],"as":["内容"]}],"cec":""
	 * }
	 */
	private void _addRule(JSONObject pJsonObj) {
		if (null != pJsonObj) {
			try {
				String lSTMName = null;
				if (pJsonObj.has("stm")) {
					lSTMName = pJsonObj.getString("stm");
				}
				// 如果流名称不为空
				if (!UTMCStringUtils.isEmpty(lSTMName)) {
					JSONArray lRules = null;
					if (pJsonObj.has("rs")) {
						lRules = pJsonObj.getJSONArray("rs");
						UTMCEventStreamGroupRuleGroup lESGR = new UTMCEventStreamGroupRuleGroup(
								lSTMName, lRules);
						if (lSTMName.equals("stm_d")
								|| lSTMName.equals("ki_stm_d")
								|| lSTMName.equals("drop")
								|| lSTMName.equals("delay")) {
							mSpecECRGRuleGroups.put(lSTMName, lESGR);
						} else {
							mECGRRuleGroups.add(lESGR);
						}
					}
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}
		}
	}

}
