package com.ut.mini.core.esg.strategy;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

public class UTMCSimpleEventIDStrategier {
	private final static String S_LEVEL_ONE = "-1";
	private final static String S_LEVEL_TWO = "-2";
	private final static String S_LEVEL_THREE = "-3";
	protected final static Object S_OPT_LOCK = new Object();
	protected Map<String, EventIDStrategyItem> mEventStrategyDict = new HashMap<String, EventIDStrategyItem>();
	private Random mRandom = new Random();

	public void addEventIDStrategy(int eventId, int pCP) {
		synchronized (S_OPT_LOCK) {
			if (pCP >= 0) {
				String lKey = "" + eventId;
				if (null != mEventStrategyDict.get(lKey)) {
					mEventStrategyDict.remove(lKey);
				}
				EventIDStrategyItem lESI = new EventIDStrategyItem();
				lESI.setEventID(eventId);
				lESI.setCP(pCP);
				mEventStrategyDict.put(lKey, lESI);
			}
		}
	}

	public UTMCStrategyResult esRandomResult(EventIDStrategyItem pESI) {
		UTMCStrategyResult lResult = new UTMCStrategyResult();
		lResult.setIsEixst(true);
		if (null != pESI) {
			if (pESI.mCP <= 100 && pESI.mCP > 0) {
				lResult.setCP(pESI.mCP);
				// Fix : nextInt返回的值有负数，比率会降低
				int lRandomValue = mRandom.nextInt(100) % 100;
				if (lRandomValue < pESI.mCP) {
					lResult.setIsOk(true);
					lResult.setCPSuccess();
					return lResult;
				}
			}
		}
		lResult.setIsOk(false);
		return lResult;
	}

	public UTMCStrategyResult isOK(int pEventId) {
		synchronized (S_OPT_LOCK) {

			if (pEventId < 0) {
				UTMCStrategyResult lResult = new UTMCStrategyResult();
				lResult.setIsOk(false);
				return lResult;
			}

			String lKey = "" + pEventId;

			// 如果存在相同ID的配置
			EventIDStrategyItem lESI = mEventStrategyDict.get(lKey);
			if (null != lESI) {
				return esRandomResult(lESI);
			}

			// 匹配100段
			int lSEventID = pEventId - (pEventId % 100);
			String lSKey = "" + lSEventID;
			lESI = mEventStrategyDict.get(lSKey);
			if (null != lESI) {
				return esRandomResult(lESI);
			}

			// 匹配1000段
			lSEventID = pEventId - (pEventId % 1000);
			lSKey = "" + lSEventID;
			lESI = mEventStrategyDict.get(lSKey);
			if (null != lESI) {
				return esRandomResult(lESI);
			}

			// 匹配20000
			if (pEventId > 20000) {
				lESI = mEventStrategyDict.get(S_LEVEL_THREE);
				if (null != lESI) {
					return esRandomResult(lESI);
				}
			}

			// 匹配10000
			if (pEventId > 10000) {
				lESI = mEventStrategyDict.get(S_LEVEL_TWO);
				if (null != lESI) {
					return esRandomResult(lESI);
				}
			}

			// 匹配全部规则
			lESI = mEventStrategyDict.get(S_LEVEL_ONE);
			if (null == lESI) { // 如果没有存在匹配的规则，那么直接放行
				UTMCStrategyResult lResult = new UTMCStrategyResult();
				lResult.setIsOk(false);
				lResult.setCP(0);
				return lResult;
			} else {
				// 如果存在-1 这个规则，那么我们需要匹配随机
				return esRandomResult(lESI);
			}
		}
	}

	class EventIDStrategyItem {
		private int mEventID;
		private int mCP;

		public int getEventID() {
			return mEventID;
		}

		public void setEventID(int pEventID) {
			this.mEventID = pEventID;
		}

		public int getCP() {
			return mCP;
		}

		public void setCP(int pCP) {
			this.mCP = pCP;
		}
	}

	public static class UTMCStrategyResult {
		private boolean mIsOk;
		private int mCP;
		private boolean mIsExist = false;
		private boolean mCPResult = false;

		public void setCPSuccess() {
			mCPResult = true;
		}

		public boolean getCPResult() {
			return mCPResult;
		}

		public boolean isOk() {
			return mIsOk;
		}

		public void setIsOk(boolean pIsOk) {
			this.mIsOk = pIsOk;
		}

		public int getCP() {
			return mCP;
		}

		public void setCP(int pCP) {
			this.mCP = pCP;
		}

		public void setIsEixst(boolean aResult) {
			mIsExist = aResult;
		}

		public boolean isExist() {
			return mIsExist;
		}

	}
}