package com.ut.mini.core.logcache;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.core.esg.UTMCEventStreamGroupBiz;
import com.ut.mini.core.esg.UTMCEventStreamGroupStrategyArrivedListener;
import com.ut.mini.core.loghelper.UTMCLogAssemble;
import com.ut.mini.librarys.UTMCRC4;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCBase64;
import com.ut.mini.utils.UTMCKeyArraySorter;
import com.ut.mini.utils.UTMCStringUtils;
import com.ut.store.UTLog;
import com.ut.store.UTLogStoreMgr;

public class UTMCGetCacheLogHelper implements UTMCEventStreamGroupStrategyArrivedListener{

	private UTMCLogCacheHelper mLogCacheHelper = null;
	private List<String> mDelayKeyCache = new LinkedList<String>();
	private Object mDelayKeyCacheOptLockObj = new Object();
	private final static int S_MAX_DELAY_ITEM_SPPED_UP_CACHE_CAPACITY = 10000; // 加速缓存的大小：10000
	private final static int MAX_PER_UPLOAD_PACKAGE_SIZE = UTMCConstants.G_MAX_PER_UPLOAD_PACKAGE_SIZE;

	public UTMCGetCacheLogHelper(UTMCLogCacheHelper aLogCacheHelper) {
		mLogCacheHelper = aLogCacheHelper;
	}

	public static class GetCacheLogResult {

		public static class CacheLogItem {
			private String mCacheKey;
			private String mLogContent;
			private List<String> mBelongStreams;

			public String getCacheKey() {
				return mCacheKey;
			}

			public void setCacheKey(String aKey) {
				this.mCacheKey = aKey;
			}

			public String getLogContent() {
				return mLogContent;
			}

			public void setLogContent(String aLogContent) {
				this.mLogContent = aLogContent;
			}

			public List<String> getBelongStreams() {
				return mBelongStreams;
			}

			public void setBelongStreams(List<String> aBelongStreams) {
				this.mBelongStreams = aBelongStreams;
			}

		}

		private List<CacheLogItem> mCacheLogItemList = new LinkedList<CacheLogItem>();
		private List<String> mDropKeyList = new LinkedList<String>();
		private List<String> mDelayKeyList = new LinkedList<String>();
		private List<String> mLogKeyList = new LinkedList<String>();
		private boolean mIsHas2001Event = false;
		private int mUploadPackageSize = 0;
		private float mPackageAvgSize = 0;
		
		public void setUploadPackageSize(int aUploadPackageSize){
			this.mUploadPackageSize = aUploadPackageSize;
		}
		
		public int getUploadPackageSize(){
			return this.mUploadPackageSize;
		}
		
		public void setPackageAvgSize(float aFloatSize){
			this.mPackageAvgSize = aFloatSize;
		}
		
		public float getPackageAvgSize(){
			return mPackageAvgSize;
		}
		
		//为兼容sqlite部分
		private List<UTLog> mLogs = new ArrayList<UTLog>();
		
		public List<UTLog> getUTLogs(){
			return mLogs;
		}
		
		public void setIsHas2001Event(){
			mIsHas2001Event = true;
		}
		
		public boolean isHas2001Event(){
			return mIsHas2001Event;
		}

		public int getCacheLogCount() {
			return mCacheLogItemList.size();
		}

		public void addLogKey(String aLogKey) {
			if (null != aLogKey) {
				mLogKeyList.add(aLogKey);
			}
		}

		public List<String> getLogKeyList() {
			return mLogKeyList;
		}

		public void addCacheLogItem(CacheLogItem aCacheLogItem) {
			if (null != aCacheLogItem) {
				mCacheLogItemList.add(aCacheLogItem);
			}
		}

		public List<CacheLogItem> getCacheLogItemList() {
			return mCacheLogItemList;
		}

		public void addDropKey(String aDropKey) {
			if (null != aDropKey) {
				this.mDropKeyList.add(aDropKey);
			}
		}

		public void addDelayKey(String aDelayKey) {
			if (null != aDelayKey) {
				this.mDelayKeyList.add(aDelayKey);
			}
		}

		public List<String> getDropKeyList() {
			return mDropKeyList;
		}

		public List<String> getDelayKeyList() {
			return mDelayKeyList;
		}

		public void addLog(UTLog log) {
			if (log!=null) {
				mLogs.add(log);
			}
		}

	}
	
	/**
	 * 为使用sqlite而改进的getCacheLog
	 * @param aNumber
	 * @param aIsCEC
	 * @param aIsASC
	 * @param aEventIDWhiteList
	 * @return
	 */
	public GetCacheLogResult getCacheLog_sqlite(int aNumber, boolean aIsCEC,boolean aIsASC,List<String> aEventIDWhiteList){
		List<UTLog> logs = UTLogStoreMgr.getInstance().get(aNumber);
		
		if(logs==null || logs.size()==0)
			return null;
		GetCacheLogResult lGCLR = new GetCacheLogResult();
		int lCurPackageSize = 0;
		int lMatchedCount = 0;
		for(UTLog log:logs){
			if (lMatchedCount >= aNumber) {
				break;
			}
			String lLogContent = log.getContent();			
			if (null != lLogContent) {
				Map<String, String> lLogMap = UTMCLogAssemble.disassemble(lLogContent);
				if (null != lLogMap) {
					List<String> lStreamNames = null;
					lStreamNames = UTMCEventStreamGroupBiz.getInstance().calStreamNames(lLogMap);

					if (lStreamNames.contains("drop")) {
						/**
						 * TODO drop 采集 先暂时修改为不采集，等待柏雍这边测试OK，再上线。
						 */
//						lStreamNames.clear();
//						lStreamNames.add("stm_d");
						lGCLR.addDropKey(log.key);
						// 加入到log中，才会被删除
						lGCLR.addLog(log);
						continue;
					}
					
					// 如果有delay的流，那么我们直接跳过
					if (lStreamNames.contains("delay")) {
						lGCLR.addDelayKey(log.key);
						continue;
					}
							
					// 如果外部设置进来有一个EventID的集合，那么我们就仅仅选中这些内容
					if(null != aEventIDWhiteList && aEventIDWhiteList.size() > 0){
						if(!aEventIDWhiteList.contains(lLogMap.get(UTLogFieldsScheme.EVENTID.toString()))){
							continue;
						}
					}

					// 如果当前包大小+日志的大小超过 最大允许的长度，那么我们跳过组包程序，直接开始上传
					if(lCurPackageSize + lLogContent.length() > MAX_PER_UPLOAD_PACKAGE_SIZE){
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "getCacheLog", "The size will exceed.");
						break;
					}
					// 累加包大小
					lCurPackageSize += lLogContent.length();
							
					lGCLR.addLogKey(log.key);
					if ("2001".equals(lLogMap.get(UTLogFieldsScheme.EVENTID.toString()))) {
						lGCLR.setIsHas2001Event();
					}

					GetCacheLogResult.CacheLogItem lCacheLogItem = new GetCacheLogResult.CacheLogItem();
					lCacheLogItem.setBelongStreams(lStreamNames);
					lCacheLogItem.setCacheKey(log.key);
					lCacheLogItem.setLogContent(lLogContent);

					lGCLR.addCacheLogItem(lCacheLogItem);
					lGCLR.addLog(log);

					lMatchedCount += 1;
				}
			 	
			}		
		
		}
		
		// 更新delay的cache key
		synchronized(mDelayKeyCacheOptLockObj){
			if(null != lGCLR.getDelayKeyList()){
				if(mDelayKeyCache.size() > S_MAX_DELAY_ITEM_SPPED_UP_CACHE_CAPACITY){
					if(UTMCLogger.isDev()){
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "delay log", "clear[size overflow:10000]");
					}
					mDelayKeyCache.clear();
				}
				mDelayKeyCache.addAll(lGCLR.getDelayKeyList());
				if(UTMCLogger.isDev()){
					UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "delay log", "add:"+lGCLR.getDelayKeyList().toString());
				}
			}
		}
					
		return lGCLR;
	}

	/**
	 * 
	 * @param aNumber			想要得到的log 数量
	 * @param aIsCEC			是否需要做eventID的兼容性，把19999的arg1替换到eventID字段。
	 * @param aIsASC			是否是升序排列
	 * @param aEventIDRangeA	是否选定给定集合的EventID数据，不再集合的不选中
	 * @return
	 */
	/*public GetCacheLogResult getCacheLog(int aNumber, boolean aIsCEC,boolean aIsASC,List<String> aEventIDWhiteList) {
		Map<String, Object> lAllCacheLog = mLogCacheHelper.getAllCacheLog();
		if (null != lAllCacheLog && lAllCacheLog.size() > 0) {
			// Log提取之后，匹配上的数量
			int lMatchedCount = 0;
			int lCurPackageSize = 0;

			GetCacheLogResult lGCLR = new GetCacheLogResult();

			String[] lMKeys = new String[lAllCacheLog.size()];
			lAllCacheLog.keySet().toArray(lMKeys);
			String[] lCacheKeys = UTMCKeyArraySorter.getInstance()
					.sortResourcesList(lMKeys, aIsASC);

			for (String lKey : lCacheKeys) {
				// 如果匹配到的数量大于等于外部给定的值，那么我们直接返回
				if (lMatchedCount >= aNumber) {
					break;
				}
				
				// 如果这个key延迟的，那么我直接继续
				synchronized(mDelayKeyCacheOptLockObj){
					if(mDelayKeyCache.contains(lKey)){
						continue;
					}
				}
				
				if (!UTMCStringUtils.isEmpty(lKey)) {
					String lContent = UTMCStringUtils
							.convertObjectToString(lAllCacheLog.get(lKey));
					if (!UTMCStringUtils.isEmpty(lContent)) {

						String lLogContent = null;
						try {
							// 还原成明文
							byte[] lBContent = UTMCBase64.decode(
									lContent.getBytes("UTF-8"),
									UTMCBase64.NO_WRAP);
							if (null != lBContent) {
								byte[] lBRc4 = UTMCRC4.rc4(lBContent,
										UTMCConstants.getRC4PrivateKey());
								lLogContent = new String(lBRc4);
							}
						} catch (Exception e) {
							e.printStackTrace();
						}

						if (null != lLogContent) {
							Map<String, String> lLogMap = UTMCLogAssemble
									.disassemble(lLogContent);
							if (null != lLogMap) {
								List<String> lStreamNames = null;
								lStreamNames = UTMCEventStreamGroupBiz
										.getInstance().calStreamNames(lLogMap);

								if (lStreamNames.contains("drop")) {
									lGCLR.addDropKey(lKey);
									continue;
								}
								// 如果有delay的流，那么我们直接跳过
								if (lStreamNames.contains("delay")) {
									lGCLR.addDelayKey(lKey);
									continue;
								}
								
								// 如果外部设置进来有一个EventID的集合，那么我们就仅仅选中这些内容
								if(null != aEventIDWhiteList && aEventIDWhiteList.size() > 0){
									if(!aEventIDWhiteList.contains(lLogMap.get(UTLogFieldsScheme.EVENTID.toString()))){
										continue;
									}
								}

								// 如果当前包大小+日志的大小超过 最大允许的长度，那么我们跳过组包程序，直接开始上传
								if(lCurPackageSize + lLogContent.length() > MAX_PER_UPLOAD_PACKAGE_SIZE){
									UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "getCacheLog", "The size will exceed.");
									break;
								}
								// 累加包大小
								lCurPackageSize += lLogContent.length();
								
								lGCLR.addLogKey(lKey);
								if ("2001".equals(lLogMap.get(UTLogFieldsScheme.EVENTID.toString()))) {
									lGCLR.setIsHas2001Event();
								}

								GetCacheLogResult.CacheLogItem lCacheLogItem = new GetCacheLogResult.CacheLogItem();
								lCacheLogItem.setBelongStreams(lStreamNames);
								lCacheLogItem.setCacheKey(lKey);
								lCacheLogItem.setLogContent(lLogContent);

								lGCLR.addCacheLogItem(lCacheLogItem);

								lMatchedCount += 1;
							}
						}
					}
				}
			}

			// 更新delay的cache key
			synchronized(mDelayKeyCacheOptLockObj){
				if(null != lGCLR.getDelayKeyList()){
					if(mDelayKeyCache.size() > S_MAX_DELAY_ITEM_SPPED_UP_CACHE_CAPACITY){
						if(UTMCLogger.isDev()){
							UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "delay log", "clear[size overflow:10000]");
						}
						mDelayKeyCache.clear();
					}
					mDelayKeyCache.addAll(lGCLR.getDelayKeyList());
					if(UTMCLogger.isDev()){
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "delay log", "add:"+lGCLR.getDelayKeyList().toString());
					}
				}
			}
			
			return lGCLR;
		}
		return null;
	}*/

	@Override
	public void onEventStreamGroupStrategyArrived() {
		synchronized(mDelayKeyCacheOptLockObj){
			mDelayKeyCache.clear();
			if(UTMCLogger.isDev()){
				UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "delay log", "clear[EventStreamGroupStrategyArrived]");
			}
		}
	}
}
