package com.ut.mini.core.logcache;

import java.util.List;
import java.util.concurrent.atomic.AtomicLong;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.module.UTMCScheduleMgr;
import com.ut.store.UTLog;
import com.ut.store.UTLogStoreMgr;


public class UTMCLogCacheHelper {

	private Context mContext = null;
//	private static Random s_random = new Random();
//	private SharedPreferences mSP = null;
//    private SharedPreferences.Editor mEditor;
    // introuduce mCount to reduce SharedPreferences.getAll().getSize() called,
    // cause getAll() will new HashMap internally.
//    private int mCount; // current key,value pairs
//    private SortedSet<String> mSortedKeys;

	private UTMCLogCacheHelper() {
	}

	/**
	 * 初始化数据，一个是存储数据的SP 如果是SDK小于9，我们就读取数据到MemCache，好用来同步数据
	 */
	private synchronized void initialize(Context pContext) {
        mContext = pContext;
	}

	public static UTMCLogCacheHelper initWithContext(Context pContext) {
		if (null != pContext) {
			UTMCLogCacheHelper lHelper = new UTMCLogCacheHelper();
			lHelper.initialize(pContext);
			return lHelper;
		}
		return null;
	}

	

	/**
	 * 缓存日志
	 * 
	 * @param pEventID
	 * @param pLogContent
	 * @param pPriority
	 *            如果超过总允许的日志条数，我们需要去清理它
	 * @return cachekey
	 *//*
	public synchronized String cacheLog(String pLogContent, String pPriority) {
		String lCacheKey = _GenerateCacheKey(pPriority);
        try {
            mEditor.putString(
                    lCacheKey,
                    new String(UTMCBase64.encode(UTMCRC4.rc4(
                            pLogContent.getBytes(),
                            UTMCConstants.getRC4PrivateKey()),
                            UTMCBase64.NO_WRAP), "UTF-8"));
            UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "cache_log",
                    pLogContent);
            mSortedKeys.add(lCacheKey);
            mCount++;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

		// 检查是否已经超标。
        int diff = mCount - UTMCConstants.G_LOG_CACHE_CAPACITY;
		if (diff > 0) {
			// 理论上是清理一条，因为都是内存操作和apply，所以无所谓啦，开销都很小
            UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "cacheLog[cache-full]",
						"start clear log, diff = " + diff);
            _clearLog(diff);
		}
		return lCacheKey;
	}*/
	
	/**
	 * 使用sqlit完成cachelog的工作
	 * @param pLogContent
	 * @param pPriority
	 * @return 为了兼容柏雍的程序，以log的id为返回值，但其实在柏雍的程序中，似乎并没有为id赋值的部分
	 */
	public synchronized String cacheLog_sqlite(String eventId,String pPriority,String index,String pLogContent ){
		UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "cache_log",pLogContent);
		List<String> streamIds = null;
		if (!TextUtils.isEmpty(eventId)) {
			UTLog utLog = new UTLog(pPriority, streamIds, eventId,pLogContent,index);
			UTLogStoreMgr.getInstance().add(utLog);
			return index;
		}else{
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "cacheLog_sqlite[event-id]", "eventid=0");
		}
		return "";
	}
	
    public synchronized void clearCacheLog_sqlite(List<UTLog> logs){
    	UTLogStoreMgr.getInstance().delete(logs);
    }

//	public synchronized Map<String, Object> getAllCacheLog() {
//        return (Map<String, Object>) mSP.getAll();
//	}

	/**
	 * 保存CacheLog到持久存储
     *
     * 调用时机：每次切到后台或者完成一次上传
	 */
	public synchronized void saveToStorage() {
//		UTMCSPHelper.fastCommit(mEditor);
//		if (UTMCLogger.isDev()) {
//			UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "saveToStorage",
//					"commit to storage");
//		}
		Runnable task = new Runnable() {
			
			@Override
			public void run() {
				UTLogStoreMgr.getInstance().store();
				
			}
		};
		UTMCScheduleMgr.getInstance().submit(task);
		
	}
}
