package com.ut.mini.core.onlineconf;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Build;

import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.onlineconf.core.UTMCOnlineConfBiz;
import com.ut.mini.utils.UTMCSPHelper;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCRealTimeDebuggingBiz extends UTMCOnlineConfBiz {

	public final static String DEBUG_API_URL = "debug_api_url";
	public final static String DEBUG_KEY = "debug_key";
	public final static String DEBUG_SAMPLING_OPTION = "debug_sampling_option";

	private static UTMCRealTimeDebuggingBiz sInstance = new UTMCRealTimeDebuggingBiz();

	public static UTMCRealTimeDebuggingBiz getInstance() {
		return sInstance;
	}

	private UTMCRealTimeDebuggingBiz() {

	}

	@Override
	public void onConfigurationArrive(String pConfName, String pConfContent) {
		// 如果在线的配置到达了:那么我直接清理掉log
		if (!UTMCStringUtils.isEmpty(pConfContent)) {
			UTMCConstants.setTransferUrl(pConfContent);
			UTMCVariables.getInstance().setRealTimeDebugFlag();
		}
	}

	@Override
	public List<String> returnRequiredConfigurationNameList() {
		List<String> lNeedConfigurations = new ArrayList<String>();
		lNeedConfigurations
				.add(UTMCConstants.OnlineConfBusiness.REAL_TIME_DEBUGGING);
		// 增加项
		return lNeedConfigurations;
	}

	// ///////////////////////////上面的通过配置控制的调试和下面的调试是相互独立的，互不影响////////////////////////

	public void turnOffRealTimeDebug() {
		UTMCConstants.setTransferUrl(UTMCConstants.G_FIXED_TRANSFER_URL);
		UTMCVariables.getInstance().resetRealTimeDebugFlag();
		UTMCVariables.getInstance().setDebugKey(null);
		
	}

	public void turnOnQuickRealTimeDebug(Map<String, String> aDebuggingMap) {
		if (null != aDebuggingMap) {
			if (aDebuggingMap.containsKey(DEBUG_API_URL)
					&& aDebuggingMap.containsKey(DEBUG_KEY)) {
				// 打开调试功能
				String lDebuggingApiUrl = aDebuggingMap.get(DEBUG_API_URL);
				String lDebugKey = aDebuggingMap.get(DEBUG_KEY);

				if (!UTMCStringUtils.isEmpty(lDebuggingApiUrl)
						&& !UTMCStringUtils.isEmpty(lDebugKey)) {

					UTMCConstants.setTransferUrl(lDebuggingApiUrl);
					UTMCVariables.getInstance().setRealTimeDebugFlag();
					UTMCVariables.getInstance().setDebugKey(lDebugKey);

				}
				
				// 如果配置了关闭采样的开关，那么我们直接穿透采集所有的数据
				if(aDebuggingMap.containsKey(DEBUG_SAMPLING_OPTION)){
					UTMCVariables.getInstance().setDebugSamplingOption();
				}
			}
		}
	}

}