package com.ut.mini.core.onlineconf;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;

import android.net.Uri;

import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.onlineconf.core.UTMCOnlineConfBiz;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCTPKBiz extends UTMCOnlineConfBiz {
	
	private String TPK_ONLINECONF_KEY = "B01N16";
	private static UTMCTPKBiz s_instance = null;
	private List<UTTPKItem> mTPKItems = new LinkedList<UTTPKItem>();
	private Map<String,String> mTPKCache = new HashMap<String,String>();
	
	private UTMCTPKBiz(){
		
	}
	
	public synchronized void addTPKItem(UTTPKItem aTPKItem){
		if(null != aTPKItem){
			mTPKItems.add(aTPKItem);
		}
	}
	
	public synchronized static UTMCTPKBiz getInstance(){
		if(null == s_instance){
			s_instance = new UTMCTPKBiz();
		}
		return s_instance;
	}

	@Override
	public synchronized void onConfigurationArrive(String pConfName, String pConfContent) {
		if(UTMCStringUtils.isEmpty(pConfContent)){
			return;
		}
        try {
        	JSONArray lTpks = new JSONArray(pConfContent);
            for (int i = 0; i < lTpks.length(); i++) {
                JSONObject lTpk = lTpks.optJSONObject(i);
                if (lTpk != null && lTpk.has("kn") && !lTpk.isNull("kn")) {
                    UTTPKItem lTPKItem = new UTTPKItem();
                    String lKn = lTpk.getString("kn");
                    String lV = lTpk.optString("v");
                    if(UTMCStringUtils.isEmpty(lV)){
                    	lV="${"+lKn+"}";
                    }
                    String lTy = lTpk.optString("ty", UTTPKItem.TYPE_FAR);
                    lTPKItem.setKname(lKn);
                    lTPKItem.setKvalue(lV);
                    lTPKItem.setType(lTy);
                    mTPKItems.add(lTPKItem);
                }
            }
        } catch (Exception e) {
        	e.printStackTrace();
        }
	}

	@Override
	public List<String> returnRequiredConfigurationNameList() {
		List<String> lConfKeyList = new ArrayList<String>(){
			/**
			 * 
			 */
			private static final long serialVersionUID = -4346629116555706556L;

			{
				add(TPK_ONLINECONF_KEY);
			}
		};
		return lConfKeyList;
	}
	
	private String _getTPKValue(String aFetchRule,Uri aPageUri,Map<String,String> aPageProperties){
		if(UTMCStringUtils.isEmpty(aFetchRule)){
			return null;
		}
		if(aFetchRule.startsWith("${url|") && aFetchRule.length() >7){
			String lRuleKey = aFetchRule.substring("${url|".length(),aFetchRule.length()-1);
			if(!UTMCStringUtils.isEmpty(lRuleKey) && null != aPageUri){
				return aPageUri.getQueryParameter(lRuleKey);
			}
		}else if(aFetchRule.startsWith("${ut|") && aFetchRule.length() >6){
			String lRuleKey = aFetchRule.substring("${ut|".length(),aFetchRule.length()-1);
			if(!UTMCStringUtils.isEmpty(lRuleKey) && null != aPageProperties){
				return aPageProperties.get(lRuleKey);
			}
		}else if(aFetchRule.startsWith("${") && aFetchRule.length() > 3){
			String lRuleKey = aFetchRule.substring("${".length(),aFetchRule.length()-1);
			if(!UTMCStringUtils.isEmpty(lRuleKey)){
				if(null != aPageProperties){
					String lValue = aPageProperties.get(lRuleKey);
					if(null != lValue){
						return lValue;
					}
				}
				if(null != aPageUri){
					return aPageUri.getQueryParameter(lRuleKey);
				}
			}
		}else{
			return aFetchRule;
		}
		return null;
	}
	
	public synchronized String getTpkString(Uri aPageUri,Map<String,String> aPageProperties){
		for(UTTPKItem lTPKItem : this.mTPKItems){
			String lKey = lTPKItem.getKname();
			String lType = lTPKItem.getType();
			String lFetchRule = lTPKItem.getKvalue();
			if(UTMCStringUtils.isEmpty(lKey)){
				return null;
			}
			String lValue = mTPKCache.get(lKey);
			if(UTMCStringUtils.isEmpty(lValue)){
				// 直接获取全新的副本
				String lNewValue = _getTPKValue(lFetchRule,aPageUri,aPageProperties);
				if(!UTMCStringUtils.isEmpty(lNewValue)){
					mTPKCache.put(lKey, lNewValue);
				}
			}else{
				if(UTTPKItem.TYPE_FAR.equals(lType)){
					continue;
				}
				String lNewValue = _getTPKValue(lFetchRule,aPageUri,aPageProperties);
				if(!UTMCStringUtils.isEmpty(lNewValue)){
					mTPKCache.put(lKey, lNewValue);
				}
			}
		}
		
		// 当外部引流的TTID存在的时候，我们需要把他放置到tpk这个内容中去。
		if(!mTPKCache.containsKey("ttid")){
			if(!UTMCStringUtils.isEmpty(UTMCVariables.getInstance().getOutsideTTID())){
				mTPKCache.put("ttid", UTMCVariables.getInstance().getOutsideTTID());
			}
		}
		
		if(mTPKCache.size()>0){
			return "{"+UTMCStringUtils.convertMapToString(mTPKCache)+"}";
		}
		return null;
	}
	
	public synchronized void sessionTimeout(){
		this.mTPKCache.clear();
	}
	
}
