
package com.ut.mini.core.onlineconf.core;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.TargetApi;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Build;
import android.os.SystemClock;

import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.request.UTMCUrlWrapper;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.module.UTMCScheduleMgr;
import com.ut.mini.plugin.UTMsgIds;
import com.ut.mini.plugin.UTPlugin;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.plugin.UTPluginMsgDispatchDelegate;
import com.ut.mini.utils.UTMCApiResponse;
import com.ut.mini.utils.UTMCBase64;
import com.ut.mini.utils.UTMCHttpUtils;
import com.ut.mini.utils.UTMCMultiProcessUtils;
import com.ut.mini.utils.UTMCSPHelper;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCOnlineConfMgr {

    private final static String CONF_CACHE_STORAGE_KEY = "conf_cache";

    private Vector<UTMCOnlineConfBiz> mConfBizList = new Vector<UTMCOnlineConfBiz>();

    /**
     * 添加配置业务模块
     * 
     * @param aBusiness 配置业务
     */
    public synchronized void addConfBiz(UTMCOnlineConfBiz pBusiness) {
        if (pBusiness != null) {
            mConfBizList.add(pBusiness);
        }
    }

    /**
     * 同步配置
     */
    public void syncOnlineConfs() {
    	
        // 再同步一把在线配置
        UTMCScheduleMgr.getInstance().submit(new SyncConfRunnable());
    }

    /**
     * 派发线上获取到的配置
     * 
     * @param configurations 配置内容
     */
    private synchronized void _dispatch(JSONObject pConfigurations) {
        // 如果SDK执行体还没有结束
        if (pConfigurations != null
                && this.mConfBizList != null) {
        	// TODO 这段代码有问题，指向了同一个json对象，如果有多个模块消耗了相同的配置名，那么这个就bug了。
        	JSONObject lConfTmp = pConfigurations;
            for (UTMCOnlineConfBiz lConfBusiness : mConfBizList) {
                if (lConfBusiness != null) {
                    // 获取模块需求的配置名称
                    List<String> lConfNameList = lConfBusiness
                            .returnRequiredConfigurationNameList();
                    // 如果配置名OK
                    if (lConfNameList != null
                            && lConfNameList.size() > 0) {
                        for (String lConfName : lConfNameList) {
                            JSONObject lContent = null;
                            if (pConfigurations.has(lConfName)) {
                                try {
                                    lContent = (JSONObject) pConfigurations
                                            .get(lConfName);
                                    lConfBusiness
                                            .onConfigurationArrive(
                                                    lConfName, lContent.getString("content"));
                                } catch (Exception e) {
                                    e.printStackTrace();
                                    continue;
                                }
                            } else {
                                try {
                                    // 没有配置时候的通知
                                    lConfBusiness.onNonConfigurationArrive(lConfName);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }
                            
                            if(lConfTmp.has(lConfName)){
                            	// 保留B01N15 保持外部也可以消费
                            	if(!"B01N15".equals(lConfName)){
                            		lConfTmp.remove(lConfName);
                            	}
                            }
                        }
                    }
                }
            }
            if(null != lConfTmp && lConfTmp.length()>0){
            	// 如果在线配置有存在自身SDK未消费的，那么我们去查查插件，看看是否有需要配置的
            	UTPluginMgr.getInstance().dispatchPluginMsg(UTMsgIds.IN_NOMATCH_ONLINE_CONF, 
            			new UTPluginMsgDispatchDelegate(lConfTmp){

					@Override
					public boolean isMatchPlugin(UTPlugin lPlugin) {
						String[] lRequiredConfNames = lPlugin.returnRequiredOnlineConfNames();
						if(null == lRequiredConfNames || lRequiredConfNames.length == 0){
							return false;
						}
						return true;
					}

					@Override
					public Object getDispatchObject(UTPlugin lPlugin) {
						String[] lRequiredConfNames = lPlugin.returnRequiredOnlineConfNames();
						JSONObject lTmpJsonObject = new JSONObject();
						JSONObject lJsonObject = (JSONObject) this.getMsgObj();
						for(String lConfName:lRequiredConfNames){
							if(lJsonObject.has(lConfName)){
								try {
									lTmpJsonObject.put(lConfName, lJsonObject.getJSONObject(lConfName));
								} catch (JSONException e) {
										e.printStackTrace();
								}
							}
						}
						if(lTmpJsonObject.length()>0){
							return lTmpJsonObject;
						}
						return null;
					}
            		
            	});
            }
        }
    }

    /**
     * 获得配置业务
     */
    public synchronized JSONObject getLocalConfCache(String pBusinessName) {
        JSONObject lGrayRelaseConfig = null;
        if (null != pBusinessName) {
            JSONObject lServerConfig = this.loadLocalConfig();
            if (null != lServerConfig) {
                try {
                    lGrayRelaseConfig = lServerConfig
                            .getJSONObject(pBusinessName);
                } catch (JSONException e) {
                }
            }
        }
        return lGrayRelaseConfig;
    }

    public synchronized void removeConfig(String pBusinessName) {
        if (null != pBusinessName) {
            JSONObject lServerConfig = this.loadLocalConfig();
            if (null != lServerConfig) {
                try {
                    lServerConfig.remove(pBusinessName);
                    this._storeServerConfig(lServerConfig);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void receiveConfig(String responseText, JSONObject localData) {
        try {
            // 把获取到的配置转换成json
            // 这里应该是取当中的一部分
            JSONObject jResult = new JSONObject(responseText);
            if (jResult != null) {
                JSONObject jData = null;
                if (jResult.has("data")) {
                    String vData = jResult.getString("data");
                    if (!UTMCStringUtils.isEmpty(vData)) {
                        // 解析配置区
                        jData = jResult.getJSONObject("data");
                    }
                }
                if (jData == null) {
                    jData = new JSONObject();
                }
                JSONObject mergedData = mergeServerConfig(jData, localData);
                _storeServerConfig(mergedData);
                // 派发
                _dispatch(mergedData);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    /**
     * 合并本地和远端配置
     */
    @SuppressWarnings("unchecked")
    private JSONObject mergeServerConfig(JSONObject pData, JSONObject pLocalData) {
        // 先合并，如果是返回gc_not_modified_304，那就用本地的，否则就用远端的
        // 如果远端删除了某个配置，本地也不要了
        try {
            JSONObject lMergedData = new JSONObject();
            for (Iterator<String> kiter = pData.keys(); kiter.hasNext();) {

                String bizK = kiter.next();
                JSONObject serverV = pData.optJSONObject(bizK);

                if (serverV != null && serverV.has("content")) {
                    JSONObject bizm = new JSONObject();

                    if ("gc_304".equals(serverV.optString("content"))) {
                        JSONObject localV = pLocalData.optJSONObject(bizK);
                        bizm.put("t", localV.getLong("t"));
                        bizm.put("content", localV.get("content"));
                    } else {
                        bizm.put("t", serverV.getLong("t"));
                        bizm.put("content", serverV.get("content"));
                    }

                    lMergedData.put(bizK, bizm);
                }

            }
            return lMergedData;
        } catch (Exception e) {
            return new JSONObject();
        }
    }

    /**
     * 保存线上推送下来的配置列表
     * 
     * @param lMergedData
     */
    @TargetApi(Build.VERSION_CODES.GINGERBREAD)
    private synchronized void _storeServerConfig(JSONObject lMergedData) {
        if (null != lMergedData) {
            Context lContext = UTMCStatConfig.getInstance().getContext();

            SharedPreferences lSp = lContext.getSharedPreferences(
                    UTMCMultiProcessUtils.getStorageName(lContext,
                            UTMCConstants.StorageKeys.G_LOG_STORAGE_CONF_NAME_PREFIS),
                    Context.MODE_PRIVATE);

            if (lSp != null) {
                Editor lEditor = lSp.edit();

                String lStoreData = lMergedData.toString();

                String lBase64Value = UTMCBase64
                        .encodeToString(lStoreData.getBytes(), UTMCBase64.NO_WRAP);
                lEditor.putString(CONF_CACHE_STORAGE_KEY, lBase64Value);
                UTMCSPHelper.fastCommit(lEditor);
            }
        }
    }

    /**
     * 加载配置
     */
    private synchronized JSONObject loadLocalConfig() {
        try {
            Context lContext = UTMCStatConfig.getInstance().getContext();

            SharedPreferences lSp = lContext.getSharedPreferences(
                    UTMCMultiProcessUtils.getStorageName(lContext,
                            UTMCConstants.StorageKeys.G_LOG_STORAGE_CONF_NAME_PREFIS),
                    Context.MODE_PRIVATE);

            if (null != lSp) {
                String lBase64ConfCache = lSp.getString(CONF_CACHE_STORAGE_KEY, "");
                if (!UTMCStringUtils.isEmpty(lBase64ConfCache)) {
                    byte[] lBConfCache = UTMCBase64.decode(lBase64ConfCache.getBytes("UTF-8"),
                            UTMCBase64.NO_WRAP);
                    if (null != lBConfCache && lBConfCache.length > 0) {
                        String lConfCache = new String(lBConfCache, "UTF-8");
                        JSONObject lConfJsonObj = new JSONObject(lConfCache);
                        return lConfJsonObj;
                    }
                }
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * 准备上传内容
     */
    private String prepareUpConfig(JSONObject pLocalData) {
        if (null == pLocalData) {
            return "";
        }
        try {
            JSONObject upData = new JSONObject();

            for (Iterator<String> kiter = pLocalData.keys(); kiter.hasNext();) {

                String bizK = kiter.next();

                JSONObject bizV = pLocalData.optJSONObject(bizK);

                // 如果包含t字段
                if (bizV != null && bizV.has("t") && bizV.optLong("t") > 0) {
                    JSONObject bizV2 = new JSONObject();
                    bizV2.put("t", bizV.getLong("t"));
                    upData.put(bizK, bizV2);
                }
            }

            return upData.toString();
        } catch (Exception e) {
            return "";
        }
    }

    private void _dispathNonConfigurationArrive() {
        for (UTMCOnlineConfBiz lConfBusiness : mConfBizList) {
            List<String> lConfNameList = lConfBusiness
                    .returnRequiredConfigurationNameList();
            // 如果配置名OK
            if (lConfNameList != null
                    && lConfNameList.size() > 0) {
                for (String lConfName : lConfNameList) {
                    lConfBusiness.onNonConfigurationArrive(lConfName);
                }
            }
        }
    }

    private class SyncConfRunnable implements Runnable {

        private int[] mRetryIntervalA = {2,2,4,4,8,8,16,16,32,64};

        @Override
        public void run() {
        	long lStartTime = SystemClock.elapsedRealtime();
        	while(true){
        		if(SystemClock.elapsedRealtime() - lStartTime > 10 * 1000){
        			break;
        		}
	            try{
		        	// 现在载入本地配置，先通知一把，防止网络IO，导致配置生效过慢的问题
		        	JSONObject lServerConfig = loadLocalConfig();
		            if (null != lServerConfig) {
		                _dispatch(lServerConfig);
		            }
		
	                // 读取304的值
	                JSONObject localData = loadLocalConfig();
	                String l304Timetamps = prepareUpConfig(localData);
	
	                Map<String, Object> postbody = new HashMap<String, Object>();
	                postbody.put("cf", l304Timetamps);
	
	                String lResponseText = "";
	                int lTryIntervalIndex = 0;
	                
	                while(UTMCStringUtils.isEmpty(lResponseText)){
	                	String lConfUrl = UTMCUrlWrapper.getSignedConfigurationUrl(UTMCConstants.G_CONF_URL, null, postbody);
	                	UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "url", lConfUrl);
	                	// 发送请求
	                	byte[] lData = UTMCHttpUtils.sendRequest(
	                        UTMCHttpUtils.HTTP_REQ_TYPE_POST_URL_ENCODED,
	                        lConfUrl, postbody, true);
	                    if (lData != null && lData.length > 0) {
		                	 try {
		                         String lResponseTextTmp = new String(lData, 0, lData.length, "UTF-8");
		                         UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "result",lResponseTextTmp);
		                         if(UTMCApiResponse.parseResult(lResponseTextTmp)){
		                        	 lResponseText = lResponseTextTmp;
		                        	 break;
		                         }
		                     } catch (UnsupportedEncodingException e) {
		                         e.printStackTrace();
		                     }
	                    }
	                    // 如果返回的内容为空，那么我们直接继续尝试
	                	if(UTMCStringUtils.isEmpty(lResponseText)){
	                		try {
                                UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "SyncConfThread", "sleep " +
                                        mRetryIntervalA[lTryIntervalIndex] * 1000 + "ms");
								Thread.sleep(mRetryIntervalA[lTryIntervalIndex]*1000);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
	                		lTryIntervalIndex += 1;
	                		if(lTryIntervalIndex >= mRetryIntervalA.length){
				                UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV, "SyncConfThread", "try end.");
	                			break;
	                		}
	                	}
	                }
	                
	            	if(!UTMCStringUtils.isEmpty(lResponseText)){
	            		receiveConfig(lResponseText, localData);
	                    UTMCVariables.getInstance().setSyncOnlineConfStatus(true);
	                    return;
	            	}
		
		
		            // 同步失败通知
		            _dispathNonConfigurationArrive();
		            
	            }catch(Throwable e){
	            	e.printStackTrace();
	            }
        	}
        	UTMCVariables.getInstance().setSyncOnlineConfStatus(true);
        }
    }

}
