package com.ut.mini.core.request;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPOutputStream;

import com.ut.mini.base.UTMCConstants;
import com.ut.mini.core.logcache.UTMCGetCacheLogHelper;
import com.ut.mini.librarys.UTMCRC4;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

public class UTMCTransferDataBuilder {
	
	public static Map<String, Object> buildPostRequestMap(
			UTMCGetCacheLogHelper.GetCacheLogResult aGCLR) {
		if (null != aGCLR) {
			// 分装流汇总数据
			Map<String, String> lReqData = new HashMap<String, String>();

			List<UTMCGetCacheLogHelper.GetCacheLogResult.CacheLogItem> lCacheLogItemList = aGCLR
					.getCacheLogItemList();

			for (UTMCGetCacheLogHelper.GetCacheLogResult.CacheLogItem lCacheLogItem : lCacheLogItemList) {
				List<String> lStreamNames = lCacheLogItem.getBelongStreams();
				for (String lSN : lStreamNames) {
					if (lReqData.containsKey(lSN)) {
						String lOLDData = lReqData.get(lSN);
						lOLDData = lOLDData + "\n"
								+ lCacheLogItem.getLogContent();
						lReqData.put(lSN, lOLDData);
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG,
								"post content", lCacheLogItem.getLogContent());
					} else {
						lReqData.put(lSN, lCacheLogItem.getLogContent());
						UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG,
								"post content", lCacheLogItem.getLogContent());
					}
				}
			}

			// 分装为post的data
			if (null != lReqData && lReqData.size() > 0) {
				int lUploadByteSize = 0;
				Map<String, Object> lBReqData = new HashMap<String, Object>();
				if (null != lReqData && lReqData.size() > 0) {
					Iterator<String> lRDKeys = lReqData.keySet().iterator();
					while (lRDKeys.hasNext()) {
						String lRDKey = lRDKeys.next();
						String lRDValue = lReqData.get(lRDKey);
						if (!UTMCStringUtils.isEmpty(lRDKey)
								&& !UTMCStringUtils.isEmpty(lRDValue)) {
							OutputStream lOutputStream = null;
							ByteArrayOutputStream lBaos = null;
							try {
								lBaos = new ByteArrayOutputStream();
								lOutputStream = new GZIPOutputStream(lBaos);
								lOutputStream.write(lRDValue.getBytes("UTF-8"));
								lOutputStream.flush();
								lOutputStream.close();
								byte[] lGZIPResult = lBaos.toByteArray();

								byte[] lBRC4ReqContent = UTMCRC4.rc4(lGZIPResult,
										UTMCConstants.getRC4PrivateKey());

								if(null != lBRC4ReqContent){
									lUploadByteSize += lBRC4ReqContent.length;
								}
								
								lBReqData.put(lRDKey, lBRC4ReqContent);
								if (UTMCLogger.isDev()) {
									UTMCLogger.i(UTMCLogger.LOG_TYPE_DEV,
											"req[stm=" + lRDKey + "]", lRDValue);
								}
							} catch (UnsupportedEncodingException e) {
								e.printStackTrace();
							} catch (Exception e) {
								e.printStackTrace();
							} finally {
								try {
									lOutputStream.close();
								} catch (Throwable e) {
									e.printStackTrace();
								}
								try {
									lBaos.close();
								} catch (Throwable e) {
									e.printStackTrace();
								}
								lBaos = null;
								lOutputStream = null;

							}
						}
					}
				}
				float lAveragePackageSize = (float) lUploadByteSize / lBReqData.size();
				UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG,"TransferDataBuilder","averagePackageSize:" + lAveragePackageSize);
				aGCLR.setUploadPackageSize(lUploadByteSize);
				aGCLR.setPackageAvgSize(lAveragePackageSize);
				return lBReqData;
			}
		}
		return null;
	}
	
}
