package com.ut.mini.core.request;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;
import java.util.Set;

import android.content.Context;

import com.ut.mini.base.IUTMCBuildInfo;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCConstants;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.UTMCDevice;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.sign.IUTRequestAuthentication;
import com.ut.mini.core.sign.UTSecuritySDKRequestAuthentication;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCBase64;
import com.ut.mini.utils.UTMCKeyArraySorter;
import com.ut.mini.utils.UTMCMD5Utils;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * 协议URL签名
 * 
 * @author daichenjie
 */
public class UTMCUrlWrapper {

	// url封装需要用到的字段
	public final static String FIELD_T = "t";
	public final static String FIELD_APPKEY = "appkey";
	public final static String FIELD_CHANNEL = "channel";
	public final static String FIELD_APPVERSION = "app_version";
	public final static String FIELD_PLATFORM = "platform";
	public final static String FIELD_SDK_VERSION = "sdk_version";
	public final static String FIELD_UTDID = "utdid";
	public final static String FIELD_V = "v";

	/**
	 * 获取签名过的上传URL
	 * 
	 * @return 签名过后的Url
	 */
	public static String getSignedTransferUrl(String pUrl,Map<String, Object> pQueryMap,Map<String, Object> pDataMap) throws Exception {
		String lSignDataStr  = "";
        if (null != pDataMap && pDataMap.size() > 0) {
            Set<String> keys = pDataMap.keySet();
            
            String[] lKeysArr = new String[keys.size()];  
            keys.toArray(lKeysArr); 
            
            lKeysArr = UTMCKeyArraySorter.getInstance().sortResourcesList(lKeysArr, true);
            
            for (String key : lKeysArr) {
                byte[] lValue = (byte[]) pDataMap.get(key);
                lSignDataStr = lSignDataStr + key + UTMCMD5Utils.getMd5Hex(lValue);
            }
        }
        String lUrl = null;
		try {
			lUrl =  _wrapUrl(pUrl,null,null,lSignDataStr);
		} catch (Exception e) {
			// 如果封装URL出现错误，那么我们就降级使用默认URL
			lUrl =  _wrapUrl(UTMCConstants.getTransferUrl(),null,null,lSignDataStr);
        }
		String lDebuggingKey = UTMCVariables.getInstance().getDebugKey();
		if(null != lDebuggingKey){
            return lUrl + "&dk=" + URLEncoder.encode(lDebuggingKey, "UTF-8");
		}
		return lUrl;
	}

	/**
	 * 获取签名过的获取配置Url
	 * 
	 * @return 签名过后的Url
	 */
	public static String getSignedConfigurationUrl(String pUrl,Map<String, Object> pQueryMap,Map<String, Object> pDataMap)
			throws Exception {
		String lUrlQueryStr = String.format("dd=%s&nsgs=1",URLEncoder.encode(_getDebugIDValue(), "UTF-8"));
        lUrlQueryStr = "dd=" + URLEncoder.encode(_getDebugIDValue(), "UTF-8") + "&nsgs=1";
		String lSignQueryStr = _getDebugIDValue();
		String lSignDataStr = "";
		
		if(pDataMap != null && pDataMap.size() > 0){
			String pCfStr = (String) pDataMap.get("cf");
			if(pCfStr != null){
				lSignDataStr = UTMCMD5Utils.getMd5Hex(pCfStr.getBytes());
			}
		}
		
		return _wrapUrl(pUrl,lUrlQueryStr,lSignQueryStr,lSignDataStr);
	}

	public static String getSignedABTestUrl(String pUrl,Map<String, Object> pQueryMap,Map<String, Object> pDataMap) throws Exception {
		String lUrlQueryStr = "";
		String lSignQueryStr = "";
		String lSignDataStr = "";
		
		if(pQueryMap != null && pQueryMap.size() > 0){
			String lLogId = (String) pQueryMap.get("logid");
			if(UTMCStringUtils.isEmpty(lLogId)){
				UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "getSignedABTestUrl", "logid is null,return abtest!");
				return null;
			}else{
                lUrlQueryStr = "logid=" + lLogId;
				lSignQueryStr = lLogId;
			}
		}
		
		return _wrapUrl(pUrl,lUrlQueryStr,lSignQueryStr,lSignDataStr);
	}

	private static String _getDebugIDValue() {
		// 这里 把 调试ID 指定为UUID，这个值，如果后期和主干代码合并的话，需要做 平台的宏定义判断
		String lDebugId = UTMCStatConfig.getInstance().getLongLoginUsernick();
		if (null != lDebugId) {
			// MD5加密
			byte[] lBMd5;
			try {
				lBMd5 = UTMCMD5Utils.getMd5(lDebugId.getBytes("UTF-8"));
				if (null != lBMd5 && lBMd5.length > 0) {
					// base64编码
					return UTMCBase64.encodeToString(lBMd5, UTMCBase64.NO_WRAP);
				}
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
		return "";
	}

	/**
	 * 封装URL
	 * 
	 * @param url
	 *            需要被封装的Url
	 * @return 封装后的Url
	 */
	private static String _wrapUrl(String url,String pUrlQueryStr,String pSignQueryStr,String pSignDataStr) throws Exception {
		Context lContext = UTMCStatConfig.getInstance().getContext();
		String lAppkey = UTMCStatConfig.getInstance().getAppkey();
		String lChannel = UTMCStatConfig.getInstance().getChannel();
		if (null == lChannel) {
			lChannel = "";
		}
		String lAppVersion = (String) UTMCDevice.getDeviceInfo(lContext).get(
				UTLogFieldsScheme.APPVERSION.toString());
		if(null != UTMCStatConfig.getInstance().getAppVersion()){
			lAppVersion = UTMCStatConfig.getInstance().getAppVersion();
		}
		String lPlatform = (String) UTMCDevice.getDeviceInfo(lContext).get(
				UTLogFieldsScheme.OS.toString());
		String lSdkVersion = UTMCConstants.G_SDK_VERSION;
		IUTMCBuildInfo lBuildInfo = UTMCStatConfig.getInstance().getBuildInfo();
		if(null != lBuildInfo){
			String lSDKVersionTmp=lBuildInfo.getShortSDKVersion();
			if(!UTMCStringUtils.isEmpty(lSDKVersionTmp)){
				lSdkVersion = lSDKVersionTmp;
			}
		}
		String lUtdid = (String) UTMCDevice.getDeviceInfo(lContext).get(
				UTLogFieldsScheme.UTDID.toString());
		String v = "3.0";// api协议版本
		String t = String.valueOf(System.currentTimeMillis());
		String lNewUrl = null;

		IUTRequestAuthentication lRequestAuthentication = UTMCStatConfig
				.getInstance().getRequestAuthenticationInstance();
		
		String lisSecureflag = "0";
		if(lRequestAuthentication instanceof UTSecuritySDKRequestAuthentication){
			lisSecureflag = "1";
		}
		
		String ltoBeSignedStr = lAppkey + lChannel + lAppVersion + lPlatform + lSdkVersion + lUtdid +
                        t + v + lisSecureflag + (pSignQueryStr==null ? "" : pSignQueryStr) +
                                (pSignDataStr==null ? "" : pSignDataStr);
		ltoBeSignedStr = UTMCMD5Utils.getMd5Hex(ltoBeSignedStr.getBytes());

		
		String lSignValue = lRequestAuthentication.getSign(ltoBeSignedStr);

		String lUrlQueryStr = "";
		if(!UTMCStringUtils.isEmpty(pUrlQueryStr)){
			lUrlQueryStr = pUrlQueryStr + "&";
		}
		lNewUrl = String.format(
				"%s?%sak=%s&av=%s&c=%s&v=%s&s=%s&d=%s&sv=%s&p=%s&t=%s&u=%s&is=%s", url,lUrlQueryStr,
				_getEncoded(lAppkey), _getEncoded(lAppVersion),
				_getEncoded(lChannel), _getEncoded(v), _getEncoded(lSignValue),
				_getEncoded(lUtdid), lSdkVersion, lPlatform, t, "",lisSecureflag);
		return lNewUrl;
	}

	/**
	 * 做 URL Encoded
	 * 
	 * @param value
	 * @return
	 */
	private static String _getEncoded(String aValue) {
		if (null == aValue) {
			return "";
		}
		try {
			return URLEncoder.encode(aValue, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return aValue;
	}

}
