package com.ut.mini.core.sign;

import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCMD5Utils;

/**
 * 基础验证模块
 * @author daichenjie
 * <pre>
 * 基于appkey+secret的签名校验方式.</p>
 * 使用方式：UTAnalytics.getInstance().setRequestAuthentication(
				new UTBaseRequestAuthentication("your app_key", "your_secret"));</p>
 * 我们也提供了更加安全的签名校验方式，请{@link UTSecuritySDKRequestAuthentication}.</p>
 * 申请地址：http://open.taobao.com.</p>
 * </pre>
 */
public class UTBaseRequestAuthentication implements IUTRequestAuthentication {

	private String mAppkey = null;
	private String mAppSecret = null;

	public String getAppkey() {
		return mAppkey;
	}

	public String getAppSecret() {
		return mAppSecret;
	}

	public UTBaseRequestAuthentication(String aAppkey, String aAppSecret) {
		mAppkey = aAppkey;
		mAppSecret = aAppSecret;
	}

	@Override
	public String getSign(String toBeSignedStr){
		if(mAppkey == null || mAppSecret == null){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG,"UTBaseRequestAuthentication", "There is no appkey,please check it!");
			return null;
		}
		
		if(toBeSignedStr == null){
			return null;
		}
		
		String lHex2 = UTMCMD5Utils.toHexString(UTMCMD5Utils
				.getMd5((toBeSignedStr + mAppSecret).getBytes()));
		return lHex2;
	}
}
