package com.ut.mini.core.sign;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

import android.content.Context;

import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.log.UTMCLogger;

/**
 * 安全SDK请求验证模块
 * @author daichenjie
 * <pre>
 * 基于阿里巴巴无线保镖安全SDK的校验方式，这个方式更加安全，我们建议您这么操作.</p>
 * 使用方式(必须先集成无线保镖服务，否则会导致无法上报数据)：UTAnalytics.getInstance().setRequestAuthentication(
				new UTSecuritySDKRequestAuthentication("your app_key"));</p>
 * 我们也提供了更加简单的签名校验方式，请{@link UTBaseRequestAuthentication}.</p>
 * 申请地址：http://open.taobao.com.</p>
 * </pre>
 */
public class UTSecuritySDKRequestAuthentication implements IUTRequestAuthentication {

	private String mAppkey = null;
	
	public String getAppkey(){
		return mAppkey;
	}
	
	public UTSecuritySDKRequestAuthentication(String aAppkey){
		mAppkey = aAppkey;
	}
	
	private Object s_securityGuardManagerObj = null;
	private Object s_secureSignatureCompObj = null;
	private Class s_securityGuardParamContextClz = null;
	private Field s_securityGuardParamContext_appKey = null;
	private Field s_securityGuardParamContext_paramMap = null;
	private Field s_securityGuardParamContext_requestType = null;
    private Method s_signRequestMethod = null;
    private int s_secureIndex = 1;
    private boolean mBInitSecurityCheck = false;

    private synchronized void _initSecurityCheck() {
    	// 如果已经校验过了，那么我们直接退出
    	if(mBInitSecurityCheck){
    		return;
    	}
        Class<?> clz = null;
        try {
            clz = Class.forName("com.taobao.wireless.security.sdk.SecurityGuardManager");
            Method lgetInstanceMethod = clz.getMethod("getInstance", Context.class);
            s_securityGuardManagerObj = lgetInstanceMethod.invoke(null, UTMCStatConfig.getInstance()
                    .getContext());
            
            Method lgetSecureSignatureComp = clz.getMethod("getSecureSignatureComp");
            s_secureSignatureCompObj = lgetSecureSignatureComp.invoke(s_securityGuardManagerObj);
        } catch (Throwable e) {
            UTMCLogger.v(UTMCLogger.LOG_TYPE_DEV, "initSecurityCheck", e.getMessage());
        }
        try {
            if (clz != null) {
            	s_securityGuardParamContextClz = Class.forName("com.taobao.wireless.security.sdk.SecurityGuardParamContext");
            	s_securityGuardParamContext_appKey = s_securityGuardParamContextClz.getDeclaredField("appKey");
            	s_securityGuardParamContext_paramMap = s_securityGuardParamContextClz.getDeclaredField("paramMap");
                s_securityGuardParamContext_requestType = s_securityGuardParamContextClz.getDeclaredField("requestType");
               
            	boolean lisThirdParty = false;
            	Method lisOpenMethod = null;
            	try{
            		 lisOpenMethod  = clz.getMethod("isOpen");
            	}catch (Throwable e) {
                    UTMCLogger.v(UTMCLogger.LOG_TYPE_DEV, "initSecurityCheck", e.getMessage());
                }
            	
            	if(lisOpenMethod != null){
            		lisThirdParty =  (Boolean) lisOpenMethod.invoke(s_securityGuardManagerObj);
            	}else{
            		Class<?> lBodyCompClz = null;
            		
            		try{
                		lBodyCompClz = Class.forName("com.taobao.wireless.security.sdk.securitybody.ISecurityBodyComponent");
            		}catch (Throwable e) {
                       UTMCLogger.v(UTMCLogger.LOG_TYPE_DEV, "initSecurityCheck", e.getMessage());
            		}
            		if(lBodyCompClz == null){
            			lisThirdParty = true;
            		}
            	}
            	
            	s_secureIndex = lisThirdParty?1:12;
            	
            	Class<?> lsignCompClz = Class.forName("com.taobao.wireless.security.sdk.securesignature.ISecureSignatureComponent");
            	s_signRequestMethod = lsignCompClz.getMethod("signRequest", s_securityGuardParamContextClz);
            }
        } catch (Throwable e) {
            UTMCLogger.v(UTMCLogger.LOG_TYPE_DEV, "initSecurityCheck", e.getMessage());
        }
		mBInitSecurityCheck = true;
    }
    
	@Override
	public String getSign(String toBeSignedStr)
	{
		
		// 如果没有初始化黑匣子相关的内容，那么我们初始化一下
		if(!mBInitSecurityCheck){
			_initSecurityCheck();
		}
		
		if(mAppkey == null){
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG,"UTSecuritySDKRequestAuthentication:getSign", "There is no appkey,please check it!");
			return null;
		}
		
		if(toBeSignedStr == null){
			return null;
		}
		
		String lSignedStr = null;
		if(s_securityGuardManagerObj != null 
				&& s_securityGuardParamContextClz != null
				&& s_securityGuardParamContext_appKey != null
				&& s_securityGuardParamContext_paramMap != null
				&& s_securityGuardParamContext_requestType != null
				&& s_signRequestMethod != null
				&& s_secureSignatureCompObj != null){
			try {
				Object lsecurityGuardParamContext = s_securityGuardParamContextClz.newInstance();
				s_securityGuardParamContext_appKey.set(lsecurityGuardParamContext, mAppkey);
				Map lParamMap = (Map) s_securityGuardParamContext_paramMap.get(lsecurityGuardParamContext);
				lParamMap.put("INPUT", toBeSignedStr);
				
				s_securityGuardParamContext_requestType.set(lsecurityGuardParamContext,s_secureIndex);
				
				lSignedStr = (String) s_signRequestMethod.invoke(s_secureSignatureCompObj, lsecurityGuardParamContext);
				
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		
		return lSignedStr;
	}

}
