package com.ut.mini.crashhandler;

import java.io.PrintWriter;
import java.io.StringWriter;

import com.ta.utdid2.android.utils.StringUtils;
import com.ut.mini.utils.UTMCMD5Utils;

public class UTExceptionParser {

	public static UTExceptionItem parse(Throwable pException) {
		if (pException != null) {

			UTExceptionItem lExceptionItem = new UTExceptionItem();

			Throwable lCause = pException.getCause();

			if (lCause == null) {
				lCause = pException;
			}
			if (lCause != null) {
				StackTraceElement[] stes = lCause.getStackTrace();
				if (stes.length > 0) {
					StackTraceElement ste = stes[0];
					if (ste != null) {
						int kPos = 0;
						String sException = lCause.toString();
						String message = "";
						StringWriter sw = new StringWriter();
						PrintWriter pw = new PrintWriter(sw);
						try {
							pException.printStackTrace(pw);
							message = sw.toString();
						} catch (Exception e) {
						} finally {
							try {
								pw.close();
								sw.close();
							} catch (Exception e) {

							}
						}
						// simple text
						if ((kPos = sException.indexOf("}:")) > 0) {
							sException = sException.substring(kPos + 2).trim();
						} else if ((kPos = sException.indexOf(":")) > 0) {
							sException = sException.substring(0, kPos);
						}

						lExceptionItem.setExpName(sException);

						if (!StringUtils.isEmpty(message)) {
							message = message.replaceAll("\n", "++");
						}

						lExceptionItem.setCrashDetail(message);

						String lMd5 = UTMCMD5Utils
								.getMd5Hex(message.getBytes());

						lExceptionItem.setMd5(lMd5);

						// 如果是SDK出错
						if (message.contains("com.taobao.statistic")
								|| message.contains("com.ut")
								|| message.contains("org.usertrack")) {
							lExceptionItem.setmCrashedByUT(true);
						} else {
							lExceptionItem.setmCrashedByUT(false);
						}
					}
				}
			}
			return lExceptionItem;
		}
		return null;
	}

	public static class UTExceptionItem {

		String mExpName = null;
		String mMd5 = null;
		String mCrashDetail = null;
		boolean mCrashedByUT = false;

		public String getExpName() {
			return mExpName;
		}

		public void setExpName(String aExpName) {
			this.mExpName = aExpName;
		}

		public String getMd5() {
			return mMd5;
		}

		public void setMd5(String aMd5) {
			this.mMd5 = aMd5;
		}

		public String getCrashDetail() {
			return mCrashDetail;
		}

		public void setCrashDetail(String aCrashDetail) {
			this.mCrashDetail = aCrashDetail;
		}

		public boolean isCrashedByUT() {
			return mCrashedByUT;
		}

		public void setmCrashedByUT(boolean mCrashedByUT) {
			this.mCrashedByUT = mCrashedByUT;
		}

	}
}