package com.ut.mini.crashhandler;

import java.lang.Thread.UncaughtExceptionHandler;
import java.util.HashMap;
import java.util.Map;

import android.content.Context;

import com.ut.mini.UTAnalytics;
import com.ut.mini.UTTracker;
import com.ut.mini.base.UTMCConstants;
import com.ut.mini.crashhandler.UTExceptionParser.UTExceptionItem;
import com.ut.mini.internal.UTOriginalCustomHitBuilder;
import com.ut.mini.log.UTMCLogger;

/**
 * 
 * @author daichenjie
 *
 */

public class UTMiniCrashHandler implements UncaughtExceptionHandler {

	private Thread.UncaughtExceptionHandler mDefaultHandler = null;
	private IUTCrashCaughtListner mListener = null;
	private Context mContext = null;
	private static UTMiniCrashHandler s_instance = new UTMiniCrashHandler();
	private final static String TAG = "UTCrashHandler";
	private boolean mIsTurnOff = false;
	private static volatile boolean mCrashing = false;

	private UTMiniCrashHandler() {

	}

	public static UTMiniCrashHandler getInstance() {
		return s_instance;
	}

	public boolean isTurnOff() {
		return mIsTurnOff;
	}

	public void turnOff() {
		if (null != mDefaultHandler) {
			Thread.setDefaultUncaughtExceptionHandler(mDefaultHandler);
			mDefaultHandler = null;
		}
		mIsTurnOff = true;
	}

	public void turnOn(Context aContext) {
		mContext = aContext;
		_initialize();
	}

	public void setCrashCaughtListener(IUTCrashCaughtListner aListener) {
		mListener = aListener;
	}

	private void _initialize() {
		if (mContext != null) {
			mDefaultHandler = Thread.getDefaultUncaughtExceptionHandler();
			Thread.setDefaultUncaughtExceptionHandler(this);
			mIsTurnOff = false;
		}
	}

	@Override
	public void uncaughtException(Thread pThread, Throwable pException) {

		try {
	        // Don't re-enter -- avoid infinite loops if crash-reporting crashes.
	        if (mCrashing) return;
	        mCrashing = true;

			if (null != pException) {
				UTMCLogger
						.e(UTMCLogger.LOG_TYPE_DEBUG, TAG,
								"Caught Exception By UTCrashHandler.Please see log as follows!");
				pException.printStackTrace();
			}

			UTExceptionItem lExceptionItem = UTExceptionParser.parse(pException);

			if (null != lExceptionItem
					&& (null != lExceptionItem.mCrashDetail
							&& null != lExceptionItem.getExpName() && null != lExceptionItem
							.getMd5())) {
	
				Map<String, String> lMap = null;
				if (null != mListener) {
					try {
						lMap = mListener.onCrashCaught(pThread, pException);
					} catch (Throwable e) {
						e.printStackTrace();
					}
				}
	
				if (null == lMap) {
					lMap = new HashMap<String, String>();
				}
	
				lMap.put("StackTrace", lExceptionItem.getCrashDetail());
	
				UTOriginalCustomHitBuilder lBuilder = new UTOriginalCustomHitBuilder(
						"UT", 1, lExceptionItem.getMd5(),
						lExceptionItem.getExpName(), null , lMap);
	
				// 设置同步发送标记
				lBuilder.setProperty(UTMCConstants.PrivateLogFields.SEND_LOG_SYNC,
						"yes");
	
				UTTracker lTracker = UTAnalytics.getInstance().getDefaultTracker();
				if(null != lTracker){
					lTracker.send(lBuilder.build());
				}else{
					UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "Record crash stacktrace error", "Fatal Error,must call setRequestAuthentication method first.");
				}
			}

		} catch (Throwable t2) {
            t2.printStackTrace();
        } finally {
        	// 链式调用外部处理crash事宜
			if (mDefaultHandler != null) {
				mDefaultHandler.uncaughtException(pThread, pException);
			}else{
				// Try everything to make sure this process goes away.
				android.os.Process.killProcess(android.os.Process.myPid());
                System.exit(10);
			}
        }
	}

}
