
package com.ut.mini.internal;

import java.util.Map;

import com.ut.device.UTDevice;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.base.UTMIVariables;
import com.ut.mini.core.UTMCLogTransfer;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.loghelper.UTMCLogAssemble;
import com.ut.mini.core.onlineconf.UTMCRealTimeDebuggingBiz;
import com.ut.mini.core.request.UTMCUrlWrapper;
import com.ut.mini.module.UTNetWorkStatusChecker;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.utils.UTMCStringUtils;

public class UTTeamWork {

	public final static String DEBUG_API_URL = UTMCRealTimeDebuggingBiz.DEBUG_API_URL;
	public final static String DEBUG_KEY = UTMCRealTimeDebuggingBiz.DEBUG_KEY;

	private static UTTeamWork s_instance = null;

	public synchronized static UTTeamWork getInstance() {
		if (null == s_instance) {
			s_instance = new UTTeamWork();
		}
		return s_instance;
	}
	
	public void initialized(){
	}

	public void turnOnRealTimeDebug(Map<String, String> aMap) {
		UTMCRealTimeDebuggingBiz.getInstance().turnOnQuickRealTimeDebug(aMap);
	}
	
	public void turnOffRealTimeDebug(){
		UTMCRealTimeDebuggingBiz.getInstance().turnOffRealTimeDebug();
	}

	/**
	 * 注册插件
	 * @param aPlugin
	 */
	public void registerPlugin(UTPlugin aPlugin){
		UTPluginMgr.getInstance().registerPlugin(aPlugin, true);
	}
	
	/**
	 * 反注册Plugin
	 * @param aPlugin
	 */
	public void unregisterPlugin(UTPlugin aPlugin) {
		UTPluginMgr.getInstance().unregisterPlugin(aPlugin);
	}
	
	public String callUTAdashAuthentication(String pUrl,Map<String, Object> pQueryMap,Map<String, Object> pDataMap) throws Exception{
		return UTMCUrlWrapper.getSignedTransferUrl(pUrl, pQueryMap, pDataMap);
	}
	/**
	 * 获取abtest 加签的url
	 * @param pUrl
	 * @param pQueryMap
	 * @param pDataMap
	 * @return
	 */
	public String getAbTestSignedUrl(String pUrl,Map<String, Object> pQueryMap,Map<String, Object> pDataMap)throws Exception{
		return UTMCUrlWrapper.getSignedABTestUrl(pUrl, pQueryMap,pDataMap) ;
	}
	
	/**
	 * 立即派发本地埋点
	 * SDK会立马启用线程去发送数据
	 */
	public void dispatchLocalHits(){
		UTMCLogTransfer.getInstance().dispatchLocalHits();
	}
	
	/**
	 * 立即保存内存中的埋点数据到本地
	 */
	public void saveCacheDataToLocal(){
		UTMCLogTransfer.getInstance().saveCacheDataToLocal();
	}
	
	/**
	 * 设置为aliyun系统,兼容wdm的app_id逻辑
	 * 警告：除了阿里云团队的产品使用之外，其他人不要调用这个方法
	 * 请在初始化方法的开始，调用这个方法，否则，会有一些先触发的日志，会是android平台。
	 */
	public void setToAliyunOsPlatform(){
		UTMIVariables.getInstance().setToAliyunOSPlatform();
	}
	
	public String getUtsid(){
		try{
			String lAppkey = UTMCStatConfig.getInstance().getAppkey();
			String lUTDID = UTDevice.getUtdid(UTMCStatConfig.getInstance().getContext());
			long lSessionTimestamp = UTMCLogAssemble.getSessionTimestamp();
			if(!UTMCStringUtils.isEmpty(lAppkey) && !UTMCStringUtils.isEmpty(lUTDID)){
				String lUtsid = lUTDID + "_" + lAppkey + "_" + lSessionTimestamp;
				return lUtsid;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}
		
	public void closeAuto1010Track(){
		UTMCVariables.getInstance().set1010AutoTrackClose();
	}
	
	public void disableNetworkStatusChecker(){
		UTNetWorkStatusChecker.disable(UTMCStatConfig.getInstance().getContext());
	}
	
}
