
package com.ut.mini.librarys;

/**
 * RC4加密/解密
 * 
 * @author daichenjie
 */
public class UTMCRC4 {

    private static class RC4Key {
        public int state[] = new int[256];
        public int x;
        public int y;
    }

    public static byte[] rc4(byte[] pData, String pPrivateKey) {
        if (null != pData && null != pPrivateKey) {
            RC4Key lRC4Key = _prepareKey(pPrivateKey);
            if (null != lRC4Key) {
                return _rc4(pData, lRC4Key);
            }
        }
        return null;
    }

    private static RC4Key _prepareKey(String pPrivateKey) {
        if (null != pPrivateKey) {
            RC4Key lRC4Key = new RC4Key();
            int index1;
            int index2;
            int counter;
            for (counter = 0; counter < 256; counter++) {
                lRC4Key.state[counter] = counter;
            }
            lRC4Key.x = 0;
            lRC4Key.y = 0;
            index1 = 0;
            index2 = 0;
            try {
                for (counter = 0; counter < 256; counter++) {
                    index2 = (pPrivateKey.charAt(index1) + lRC4Key.state[counter] + index2) % 256;
                    int swapInt = lRC4Key.state[counter];
                    lRC4Key.state[counter] = lRC4Key.state[index2];
                    lRC4Key.state[index2] = swapInt;
                    index1 = (index1 + 1) % pPrivateKey.length();
                }
            } catch (Exception e) {
                return null;
            }
            return lRC4Key;
        }
        return null;
    }

    private static byte[] _rc4(byte[] pData, RC4Key pKey) {
        if (null != pData && null != pKey) {
            int x;
            int y;
            int counter;
            int xorIndex;
            x = pKey.x;
            y = pKey.y;

            for (counter = 0; counter < pData.length; counter++) {
                // 这里的代码都是以int为基础
                x = (x + 1) % 256;
                y = (pKey.state[x] + y) % 256;
                int lSwapInt = pKey.state[x];
                pKey.state[x] = pKey.state[y];
                pKey.state[y] = lSwapInt;
                xorIndex = (pKey.state[x] + pKey.state[y]) % 256;
                // 这句话很关键，是byte的和int的做位运算，运算的时候是int的精度，运算之后，会转换成byte，精度没有变化。故可行
                pData[counter] ^= pKey.state[xorIndex];
            }
            pKey.x = x;
            pKey.y = y;
            return pData;
        }
        return null;
    }

}
