
package com.ut.mini.log;

import android.util.Log;

import com.ut.mini.plugin.UTPluginContext;
import com.ut.mini.plugin.UTPluginMgr;
import com.ut.mini.utils.UTMCStringUtils;

/**
 * Log输出
 * 
 * @author daichenjie
 */
public class UTMCLogger {

    public final static int LOG_TYPE_DEBUG = 1;
    public final static int LOG_TYPE_DEV = 2;
    private final static int LOG_LEVEL_V = 1;
    private final static int LOG_LEVEL_D = 2;
    private final static int LOG_LEVEL_I = 3;
    private final static int LOG_LEVEL_W = 4;
    private final static int LOG_LEVEL_E = 5;
    private static String s_log_prefix = "utmini:";
    private static boolean s_is_dev = false;
    private static boolean s_is_debug = false;

    public static boolean isDev() {
        return s_is_dev;
    }

    public static void setDev(boolean isDev) {
        UTMCLogger.s_is_dev = isDev;
    }

    public static void setLogPrefix(String pLogPrefix) {
        s_log_prefix = pLogPrefix;
    }

    public static boolean isDebug() {
        return s_is_debug;
    }

    public static void setDebug(boolean isDebug) {
        UTMCLogger.s_is_debug = isDebug;
        UTPluginMgr.getInstance().updatePluginContextValue(UTPluginContext.DEBUG_LOG_SWITCH);
    }

    private static void _printLog(int logType, int level, String tag,
            Object content) {
        String sContent = UTMCStringUtils.convertObjectToString(content);
        if (UTMCStringUtils.isEmpty(sContent) || UTMCStringUtils.isEmpty(tag)) {
            return;
        }
        if (logType == LOG_TYPE_DEBUG) {
            if (!(s_is_dev || s_is_debug)) {
                return;
            }
        } else if (logType == LOG_TYPE_DEV) {
            if (!s_is_dev) {
                return;
            }
        }
        sContent = tag + ":::" + sContent;
    	tag="UTMini";
        switch (level) {
            case LOG_LEVEL_V:
                Log.v(tag, sContent);
                break;
            case LOG_LEVEL_D:
                Log.d(tag, sContent);
                break;
            case LOG_LEVEL_I:
                Log.i(tag, sContent);
                break;
            case LOG_LEVEL_W:
                Log.w(tag, sContent);
                break;
            case LOG_LEVEL_E:
                Log.e(tag, sContent);
                break;
        }
    }

    public static void v(int logType, String tag, Object content) {
        _printLog(logType, LOG_LEVEL_V, tag, content);
    }

    public static void d(int logType, String tag, Object content) {
        _printLog(logType, LOG_LEVEL_D, tag, content);
    }

    public static void i(int logType, String tag, Object content) {
        _printLog(logType, LOG_LEVEL_I, tag, content);
    }

    public static void w(int logType, String tag, Object content) {
        _printLog(logType, LOG_LEVEL_W, tag, content);
    }

    public static void e(int logType, String tag, Object content) {
        _printLog(logType, LOG_LEVEL_E, tag, content);
    }

}
