package com.ut.mini.module;

import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public class UTMCScheduleMgr {

	private ScheduledExecutorService mScheduledES = null;

	private UTMCScheduleMgr() {
		mScheduledES = Executors.newScheduledThreadPool(3,new ThreadFactory(){
			
			 private final AtomicInteger mCount = new AtomicInteger(1);

             @Override public Thread newThread(final Runnable r) {
                 return new Thread(r, "UTScheduleMgr #" + mCount.getAndIncrement());
             }
			
		});
	}

	private static UTMCScheduleMgr s_instance = null;

	public synchronized static UTMCScheduleMgr getInstance() {
		if (null == s_instance) {
			s_instance = new UTMCScheduleMgr();
		}
		return s_instance;
	}

	public Future<?> submit(Callable<?> aTask) {
		return mScheduledES.submit(aTask);
	}

	public Future<?> submit(Runnable aTask) {
		return mScheduledES.submit(aTask);
	}

	public ScheduledFuture<?> scheduleAtFixedRate(Runnable aCommand,
			long aInitialDelay, long aPeriod, TimeUnit aUnit) {
		return mScheduledES.scheduleAtFixedRate(aCommand, aInitialDelay,
				aPeriod, aUnit);
	}
	
	public ScheduledFuture<?> scheduleWithFixedDelay(Runnable aCommand,
			long aInitialDelay, long aPeriod, TimeUnit aUnit) {
		return mScheduledES.scheduleWithFixedDelay(aCommand, aInitialDelay,
				aPeriod, aUnit);
	}
	
	public ScheduledFuture<?> schedule(Runnable aCommand,long aDelay,TimeUnit aUnit) {
		return mScheduledES.schedule(aCommand, aDelay, aUnit);
	}
	
	public ScheduledFuture<?> schedule(Callable<?> aCallable,long aDelay,TimeUnit aUnit) {
		return mScheduledES.schedule(aCallable, aDelay, aUnit);
	}
	
	public void execute(Runnable aRunnable) {
		mScheduledES.execute(aRunnable);
	}
	
}
