package com.ut.mini.plugin;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import android.app.Activity;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;

import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.appstatus.UTMCAppStatusCallbacks;
import com.ut.mini.core.appstatus.UTMCAppStatusRegHelper;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.utils.UTMCStringUtils;

public class UTPluginMgr implements UTMCAppStatusCallbacks {

	public final static String PARTNERPLUGIN_UTPREF = "com.ut.mini.perf.UTPerfPlugin";
	private final static int SEND_PLUGIN_MSG = 1;
	private static UTPluginMgr s_instance = new UTPluginMgr();
	private HandlerThread mHandlerThread = null;
	private Handler mHandler = null;
	// 消息同步派发的列表，在这里控制可靠.
	// 唯独Crash的消息同步派发给所有的插件.
	private List<UTPlugin> mDispatchMsgSyncList = new LinkedList<UTPlugin>();
	private List<String> mPartnerPluginExistList = new ArrayList<String>();
	
	private List<String> mPartnerPluginList = new ArrayList<String>(){
		/**
		 * 
		 */
		private static final long serialVersionUID = -8811681421253046664L;

		{
			add(PARTNERPLUGIN_UTPREF);// 柏雍这边的性能数据采集插件
//			add("com.ut.minisdk.UTTestPlugin");
		}
	};

	private UTPluginMgr() {
		// 如果是系统4.0到以上，我们让插件具备接收前后台通知的能力
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
			UTMCAppStatusRegHelper.registerAppStatusCallbacks(this);
		}
	}

	public static UTPluginMgr getInstance() {
		return s_instance;
	}

	private void _startDispathMsgThread() {
		mHandlerThread = new HandlerThread("UT-PLUGIN-ASYNC");
		mHandlerThread.start();
		mHandler = new Handler(mHandlerThread.getLooper()) {

			@Override
			public void handleMessage(Message msg) {
				if (msg.what == SEND_PLUGIN_MSG) {
					if (msg.obj instanceof UTPluginMsgItem) {
						UTPluginMsgItem lPluginMsgItem = (UTPluginMsgItem) msg.obj;
						UTPlugin lPlugin = lPluginMsgItem.getPluginObj();
						int lMsgId = lPluginMsgItem.getMsgId();
						Object lMsgObj = lPluginMsgItem.getMsgObj();
						if (null != lPlugin) {
							try {
								if(lMsgObj instanceof UTPluginMsgDispatchDelegate){
									UTPluginMsgDispatchDelegate lDispatchDelegate = (UTPluginMsgDispatchDelegate)lMsgObj;
									if(lDispatchDelegate.isMatchPlugin(lPlugin)){
										// 如果插件是希望派发的对象，那么，我们直接获取派发对象，来完成派发的目的
										Object lObject = lDispatchDelegate.getDispatchObject(lPlugin);
										lPlugin.onPluginMsgArrivedFromSDK(lMsgId, lObject);
									}
								}else{
									lPlugin.onPluginMsgArrivedFromSDK(lMsgId, lMsgObj);
								}
							} catch (Throwable e) {
								e.printStackTrace();
							}
						}
					}
				}
			}

		};
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean isPartnerPluginExist(String aPluginClassPath){
		if(mPartnerPluginExistList.contains(aPluginClassPath)){
			return true;
		}
		return false;
	}
	
	private synchronized void _updatePluginContextValueTraversal(int aPluginContextKey,UTPluginContextValueDispatchDelegate aDeledate){
		if(null == aDeledate){
			return;
		}
		for(UTPlugin lPlugin:mPlugins){
			aDeledate.onPluginContextValueChange(lPlugin.getPluginContext());
			lPlugin.onPluginContextValueUpdate(aPluginContextKey);
		}
	}
	
	public void updatePluginContextValue(int aPluginContextKey){
		switch(aPluginContextKey){
		case UTPluginContext.DEBUG_LOG_SWITCH:
			_updatePluginContextValueTraversal(aPluginContextKey,new UTPluginContextValueDispatchDelegate(){

				@Override
				public void onPluginContextValueChange(
						UTPluginContext aOriginalPluginContext) {
					aOriginalPluginContext.setDebugLogFlag(UTMCLogger.isDebug());
				}
				
			});
			break;
		default:
			break;
		}
	}
	
	/**
	 * 启动合作方插件
	 */
	public void runPartnerPlugin(){
		if(null != mPartnerPluginList && mPartnerPluginList.size()>0){
			for(String lPluginClassPath : mPartnerPluginList){
				if(!UTMCStringUtils.isEmpty(lPluginClassPath)){
					try {
						Class<?> lPluginClass = Class.forName(lPluginClassPath);
						Object lPluginObject = lPluginClass.newInstance();
						if(lPluginObject instanceof UTPlugin){
							this.registerPlugin((UTPlugin)lPluginObject, true);
							UTMCLogger.i(UTMCLogger.LOG_TYPE_DEBUG, "UTPluginMgr", "runPartnerPlugin[OK]:"+lPluginClassPath);
							mPartnerPluginExistList.add(lPluginClassPath);
						}
					} catch (ClassNotFoundException e) {
//						e.printStackTrace();
					} catch (InstantiationException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	private List<UTPlugin> mPlugins = new LinkedList<UTPlugin>();

	private UTPluginContext _getPluginContext(){
		UTPluginContext lPluginContext = new UTPluginContext();
		lPluginContext.setContext(UTMCStatConfig.getInstance().getContext());
		if(UTMCLogger.isDebug()){
			lPluginContext.setDebugLogFlag(UTMCLogger.isDebug());;
		}
		return lPluginContext;
	}
	
	public synchronized void registerPlugin(UTPlugin aPlugin,
			boolean aAsyncMode) {
		if (null != aPlugin && !mPlugins.contains(aPlugin)) {
			aPlugin.setPluginContext(_getPluginContext());
			mPlugins.add(aPlugin);
			if (!aAsyncMode) {
				mDispatchMsgSyncList.add(aPlugin);
			}
			//有且只调用一次
			aPlugin.onRegistered();
		}
	}

	public synchronized void unregisterPlugin(UTPlugin aPlugin) {
		if (null != aPlugin && mPlugins.contains(aPlugin)) {
			mPlugins.remove(aPlugin);
			//有且只调用一次
			aPlugin.onUnRegistered();
			aPlugin.setPluginContext(null);
		}
		if (null != mDispatchMsgSyncList
				&& mDispatchMsgSyncList.contains(aPlugin)) {
			mDispatchMsgSyncList.remove(aPlugin);
		}
	}

	private boolean _isMsgIdInA(int aMsgId, int[] aMsgA) {
		boolean lIsFound = false;
		if (null != aMsgA) {
			for (int lId : aMsgA) {
				if (lId == aMsgId) {
					lIsFound = true;
				}
			}
		}
		return lIsFound;
	}

	public synchronized boolean dispatchPluginMsg(int aMsgId, Object aMsgObj) {
		if (null == mHandler) {
			this._startDispathMsgThread();
		}
		boolean lIsDispatched = false;
		if (mPlugins.size() > 0) {
			for (UTPlugin lPlugin : mPlugins) {
				int[] lMsgIds = lPlugin.returnRequiredMsgIds();
				if (null != lMsgIds && _isMsgIdInA(aMsgId, lMsgIds)) {
					// 如果是crash消息，消息是在同步的列表之内，那么我们同步派发给插件
					if (aMsgId == UTMsgIds.IN_CRASH_OCCOUR
							|| (null != mDispatchMsgSyncList && mDispatchMsgSyncList
									.contains(lPlugin))) {
						try {
							if(aMsgObj instanceof UTPluginMsgDispatchDelegate){
								UTPluginMsgDispatchDelegate lDispatchDelegate = (UTPluginMsgDispatchDelegate)aMsgObj;
								if(lDispatchDelegate.isMatchPlugin(lPlugin)){
									// 如果插件是希望派发的对象，那么，我们直接获取派发对象，来完成派发的目的
									Object lObject = lDispatchDelegate.getDispatchObject(lPlugin);
									lPlugin.onPluginMsgArrivedFromSDK(aMsgId, lObject);
								}
							}else{
								lPlugin.onPluginMsgArrivedFromSDK(aMsgId, aMsgObj);
							}
							lIsDispatched = true;
						} catch (Throwable e) {
							e.printStackTrace();
						}
					} else {
						// 异步派发给插件
						UTPluginMsgItem lMsgItem = new UTPluginMsgItem();
						lMsgItem.setMsgId(aMsgId);
						lMsgItem.setMsgObj(aMsgObj);
						lMsgItem.setPluginObj(lPlugin);
						// 异步发送
						Message lMessage = Message.obtain();
						lMessage.what = SEND_PLUGIN_MSG;
						lMessage.obj = lMsgItem;
						mHandler.sendMessage(lMessage);
						lIsDispatched = true;
					}
				}
			}
		}
		return lIsDispatched;
	}

	private static class UTPluginMsgItem {
		private int mMsgId = 0;
		private Object mMsgObj = null;
		private UTPlugin mPluginObj = null;

		public int getMsgId() {
			return mMsgId;
		}

		public void setMsgId(int aMsgId) {
			this.mMsgId = aMsgId;
		}

		public Object getMsgObj() {
			return mMsgObj;
		}

		public void setMsgObj(Object aMsgObj) {
			this.mMsgObj = aMsgObj;
		}

		public UTPlugin getPluginObj() {
			return mPluginObj;
		}

		public void setPluginObj(UTPlugin aPluginObj) {
			this.mPluginObj = aPluginObj;
		}
	}

	@Override
	public void onSwitchBackground() {
		this.dispatchPluginMsg(UTMsgIds.IN_SWITCH_BACKGROUND, null);
	}

	@Override
	public void onSwitchForeground() {
		this.dispatchPluginMsg(UTMsgIds.IN_SWITCH_FOREGROUND, null);
	}

	@Override
	public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
		
	}

	@Override
	public void onActivityDestroyed(Activity activity) {
		
	}

	@Override
	public void onActivityPaused(Activity activity) {
		
	}

	@Override
	public void onActivityResumed(Activity activity) {
		
	}

	@Override
	public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
		
	}

	@Override
	public void onActivityStarted(Activity activity) {
		
	}

	@Override
	public void onActivityStopped(Activity activity) {
		
	}

}
