package com.ut.mini.sdkevents;

import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Context;
import android.os.Bundle;
import android.os.SystemClock;

import com.ut.mini.UTAnalytics;
import com.ut.mini.UTInterfaceCallDelegate;
import com.ut.mini.UTPageHitHelper;
import com.ut.mini.UTTracker;
import com.ut.mini.base.UTLogFieldsScheme;
import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.core.UTMCVariables;
import com.ut.mini.core.appstatus.UTMCAppStatusCallbacks;
import com.ut.mini.internal.UTOriginalCustomHitBuilder;
import com.ut.mini.log.UTMCLogger;
import com.ut.mini.plugin.UTMsgIds;
import com.ut.mini.plugin.UTPlugin;

public class UTMI1010_2001Event extends UTPlugin implements UTMCAppStatusCallbacks {

	private long mHowLongForegroundStay = 0;
	private long mToForegroundTimestamp = 0l;
	private long mToBackgroundTimestamp = 0l;
	
	@Override
	public int[] returnRequiredMsgIds() {
		int[] lMsgIdA = {UTMsgIds.IN_DISPATCH_SIMPLE_LOG_MAP};
		return lMsgIdA;
	}

	/**
	 * 这个功能只会在4.0以下，会监听MINICore的页面事件打点，累计页面2001事件的arg3字段作为1010的时长 切换到后台，自动清理
	 */
	@Override
	public void onPluginMsgArrivedFromSDK(int aInPluginMsgId, Object aMsgObject) {
		if (aInPluginMsgId == UTMsgIds.IN_DISPATCH_SIMPLE_LOG_MAP) {
			Map<String, String> lLogMap = (Map<String, String>) aMsgObject;
			if (lLogMap.containsKey(UTLogFieldsScheme.EVENTID.toString())) {
				String lEventId = lLogMap.get(UTLogFieldsScheme.EVENTID.toString());
				// 如果征听到2001，2002的打点，那么我们就检测是否切换到了后台，这里使用GetTask权限来做
				if ("2001".equals(lEventId)) {

					long lDuration = 0;
					if (lLogMap.containsKey(UTLogFieldsScheme.ARG3.toString())) {
						String lDurationStr = lLogMap
								.get(UTLogFieldsScheme.ARG3.toString());
						try {
							lDuration = Long.parseLong(lDurationStr);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}

					mHowLongForegroundStay += lDuration;

					if (_isSwitchBackgroundByGetTask()) {
						// 发送1010 打点
						_send1010Hit(mHowLongForegroundStay);
						// 输出1010打点
						mHowLongForegroundStay = 0l;
					}
				}
			}
		}
	}
	
	/**
	 * 这个方法，我考虑一下，是否可以挪到另外的地方去实现（utils中），尼玛这个很坑爹啊。
	 * 
	 * @return
	 */
	private static boolean _isSwitchBackgroundByGetTask() {
		try {
			Context lContext = UTMCStatConfig.getInstance().getContext();
			if (lContext != null) {
				String lPackageName = lContext.getPackageName();
				if (null != lPackageName) {
					ActivityManager manager = (ActivityManager) lContext
							.getSystemService(Activity.ACTIVITY_SERVICE);
					if (manager != null) {
						try {
							List<ActivityManager.RunningTaskInfo> task_info = manager
									.getRunningTasks(1);
							if (task_info != null) {
								if (task_info.size() > 0) {
									ComponentName cn = task_info.get(0).topActivity;
									if (null != cn
											&& lPackageName.contains(cn
													.getPackageName())) {
										return false;
									}
								}
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	private void _send1010Hit(long aHowLongForegroundStay) {
		// 允许外部关闭1010的自动采集【戴码：2015年 6月16日 星期二 10时49分15秒 CST】
		if(!UTMCVariables.getInstance().is1010AutoTrackClosed()){
			if (aHowLongForegroundStay > 0) {
				long lStayBackgroundDuration = 0l;
				if(0 != mToBackgroundTimestamp){
					lStayBackgroundDuration = SystemClock.elapsedRealtime() - mToBackgroundTimestamp;
				}
				UTOriginalCustomHitBuilder lHitBuilder = new UTOriginalCustomHitBuilder(
						"UT", 1010, "" + aHowLongForegroundStay, ""+lStayBackgroundDuration, null, null);
				UTTracker lTracker = UTAnalytics.getInstance().getDefaultTracker();
				if(null != lTracker){
					lTracker.send(lHitBuilder.build());
				}else{
					UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, "Record app display event error", "Fatal Error,must call setRequestAuthentication method first.");
				}
			}
		}
	}

	/**
	 * 在4.0以上，我们利用系统的activity机制，来实现时长的计算，后台-前台
	 */
	@Override
	public void onSwitchBackground() {
		long lHowLongForegroundStay = SystemClock.elapsedRealtime()
				- mToForegroundTimestamp;
		_send1010Hit(lHowLongForegroundStay);
		mToBackgroundTimestamp = SystemClock.elapsedRealtime();
	}

	@Override
	public void onSwitchForeground() {
		mToForegroundTimestamp = SystemClock.elapsedRealtime();
	}

	@Override
	public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
		
	}

	@Override
	public void onActivityDestroyed(Activity activity) {
		
	}

	@Override
	public void onActivityPaused(Activity activity) {
		UTInterfaceCallDelegate.pageDisAppearByAuto(activity);
	}

	@Override
	public void onActivityResumed(Activity activity) {
		UTInterfaceCallDelegate.pageAppearByAuto(activity);
	}

	@Override
	public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
		
	}

	@Override
	public void onActivityStarted(Activity activity) {
		
	}

	@Override
	public void onActivityStopped(Activity activity) {
		
	}

}
