package com.ut.mini.utils;

import java.util.Arrays;
import java.util.Comparator;

/**
 * Key数组排序器
 * @author daichenjie
 * 这里可以选择，ASC，DESC来排序我们需要的数据
 */
public class UTMCKeyArraySorter {
	
	private static UTMCKeyArraySorter s_instance = null;
	private ResourcesDESCComparator mDESCComparator = new ResourcesDESCComparator();
	private ResourcesASCComparator mASCComparator = new ResourcesASCComparator();
	
	private UTMCKeyArraySorter(){
		
	}
	
	public synchronized static UTMCKeyArraySorter getInstance(){
		if(null == s_instance){
			s_instance = new UTMCKeyArraySorter();
		}
		return s_instance;
	}
	
	public String[] sortResourcesList(String[] resources,
			boolean aUseASC) {
		Comparator<String> lCompare = null;
		if(aUseASC){
			lCompare  = mASCComparator;
		}else{
			lCompare = mDESCComparator;
		}
		if (null != lCompare && resources != null && resources.length > 0) {
			Arrays.sort(resources, lCompare);
			return resources;
		}
		return null;
	}

	// 降序排列
	private class ResourcesDESCComparator implements
			Comparator<String> {

		@Override
		public int compare(String o1, String o2) {
			if (!UTMCStringUtils.isEmpty(o1)
					&& !UTMCStringUtils.isEmpty(o2)) {
				int i = o1.compareTo(o2);
				return -1 * i;
			}
			return 0;
		}
	}

	// 升序排列
	private class ResourcesASCComparator implements
			Comparator<String> {

		@Override
		public int compare(String o1, String o2) {
			if (!UTMCStringUtils.isEmpty(o1)
					&& !UTMCStringUtils.isEmpty(o2)) {
				return o1.compareTo(o2);
			}
			return 0;
		}
	}
}
