package com.ut.mini.utils;

import android.content.Context;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.telephony.TelephonyManager;

public class UTMCNetworkUtils {
	
	
	public static final String	NETWORK_CLASS_WIFI		= "wifi";
	public static final String	NETWORK_CLASS_2_G		= "2G";
	public static final String	NETWORK_CLASS_3_G		= "3G";
	public static final String	NETWORK_CLASS_4_G		= "4G";
	public static final String	NETWORK_CLASS_UNKNOWN	= "Unknown";

	public static String getNetworkType(Context aContext) {
	
		Context context = aContext;
		if (context == null) {
			return NETWORK_CLASS_UNKNOWN;
		}
		try {
			PackageManager pManager = context.getPackageManager();
			if (pManager.checkPermission("android.permission.ACCESS_NETWORK_STATE", context.getPackageName()) != 0) {
				return NETWORK_CLASS_UNKNOWN;
			}
			ConnectivityManager cManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo nInfo = cManager.getActiveNetworkInfo();
			if (nInfo == null) {
				return NETWORK_CLASS_UNKNOWN;
			}
			if (nInfo.isConnected()) {
				if (nInfo.getType() == ConnectivityManager.TYPE_WIFI) {
					return NETWORK_CLASS_WIFI;
				} else if (nInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
					return getNetworkClass(nInfo.getSubtype());
				}
			}
		} catch (Throwable e) {
		}

		return NETWORK_CLASS_UNKNOWN;
	}

	public static boolean isConnected(Context aContext) {
		Context context = aContext;
		if (context != null) {
			try {
				ConnectivityManager conManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
				if (conManager != null) {
					NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
					if (networkInfo != null) {
						return networkInfo.isConnected();
					} else {
						// 没有网络的时候，是没有activityNetworkInfo，这个测试通过
						return false;
					}
				}
			} catch (Exception e) {
			}
		}
		return true;
	}

	private static String getNetworkClass(int networkType) {
		switch (networkType) {
		case TelephonyManager.NETWORK_TYPE_GPRS:
		case TelephonyManager.NETWORK_TYPE_EDGE:
		case TelephonyManager.NETWORK_TYPE_CDMA:
		case TelephonyManager.NETWORK_TYPE_1xRTT:
		case TelephonyManager.NETWORK_TYPE_IDEN:
			return NETWORK_CLASS_2_G;
		case TelephonyManager.NETWORK_TYPE_UMTS:
		case TelephonyManager.NETWORK_TYPE_EVDO_0:
		case TelephonyManager.NETWORK_TYPE_EVDO_A:
		case TelephonyManager.NETWORK_TYPE_HSDPA:
		case TelephonyManager.NETWORK_TYPE_HSUPA:
		case TelephonyManager.NETWORK_TYPE_HSPA:
		case TelephonyManager.NETWORK_TYPE_EVDO_B:
		case TelephonyManager.NETWORK_TYPE_EHRPD:
		case TelephonyManager.NETWORK_TYPE_HSPAP:
			return NETWORK_CLASS_3_G;
		case TelephonyManager.NETWORK_TYPE_LTE:
			return NETWORK_CLASS_4_G;
		default:
			return NETWORK_CLASS_UNKNOWN;
		}
	}

	/**
     * 检查网络是否通畅
     * 
     * @param pContext
     * @return 网络互通用的是internet权限，这里检查用的是ACCESS_NETWORK_STATE
     *         权限，所以，默认我们需要返回true，只有真的确认网络没有，再返回
     */
    public static boolean isConnectInternet(Context pContext) {
        if (pContext != null) {
            try {
                ConnectivityManager conManager = (ConnectivityManager) pContext
                        .getSystemService(Context.CONNECTIVITY_SERVICE);
                if (conManager != null) {
                    PackageManager localPackageManager = pContext
                            .getPackageManager();

                    if (localPackageManager.checkPermission(
                            "android.permission.ACCESS_NETWORK_STATE",
                            pContext.getPackageName()) == PackageManager.PERMISSION_GRANTED) {
                        NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
                        if (networkInfo != null) {
                            return networkInfo.isConnected();
                        }else{
                            // 没有网络的时候，是没有activityNetworkInfo，这个测试通过 
                            return false;
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return true;
    }

    public static String[] getNetworkState(Context paramContext) {
        String[] arrayOfString = {
                "Unknown", "Unknown"
        };
        try {
            PackageManager localPackageManager = paramContext
                    .getPackageManager();
            if (localPackageManager.checkPermission(
                    "android.permission.ACCESS_NETWORK_STATE",
                    paramContext.getPackageName()) != 0) {
                arrayOfString[0] = "Unknown";
                return arrayOfString;
            }
            ConnectivityManager localConnectivityManager = (ConnectivityManager) paramContext
                    .getSystemService("connectivity");
            if (localConnectivityManager == null) {
                arrayOfString[0] = "Unknown";
                return arrayOfString;
            }
            NetworkInfo localNetworkInfo1 = localConnectivityManager
                    .getNetworkInfo(1);
            if (localNetworkInfo1 != null
                    && localNetworkInfo1.getState() == NetworkInfo.State.CONNECTED) {
                arrayOfString[0] = "Wi-Fi";
                return arrayOfString;
            }
            NetworkInfo localNetworkInfo2 = localConnectivityManager
                    .getNetworkInfo(0);
            if (localNetworkInfo2 != null
                    && localNetworkInfo2.getState() == NetworkInfo.State.CONNECTED) {
                arrayOfString[0] = "2G/3G";
                arrayOfString[1] = localNetworkInfo2.getSubtypeName();
                return arrayOfString;
            }
        } catch (Exception e) {

        }
        return arrayOfString;
    }

    public static String getWifiAddress(Context context) {
        if (context != null) {
          WifiManager wifimanage = (WifiManager)context
            .getSystemService("wifi");
          WifiInfo wifiinfo = wifimanage.getConnectionInfo();
          if (wifiinfo != null) {
            String address = wifiinfo.getMacAddress();
            if (UTMCStringUtils.isEmpty(address)) {
              address = "00:00:00:00:00:00";
            }
            return address;
          }
          return "00:00:00:00:00:00";
        }

        return "00:00:00:00:00:00";
      }
    
	
}
