package com.ut.store;

import java.util.ArrayList;
import java.util.List;

import android.annotation.SuppressLint;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Build;

import com.ut.mini.log.UTMCLogger;

/**
 * 从柏雍的程序迁移过来，添加了与utmini兼容的key项 所有与FIELD_NAME_KEY相关的都是新添加的
 * 
 * @author gelin.gl
 * 
 */
public class UTLogSqliteStore implements IUTLogStore {

	private static final String	TAG								= "UTSqliteLogStore";
	private static final String	DATABASE_NAME					= "usertrack.db";
	private static final int	DATABASE_VERSION				= 1;
	private static final String	TABLE_NAME						= "log";
	private static final String	CREATE_TABLE_IF_NO_EXIST_SQL	= "CREATE TABLE IF NOT EXISTS " + TABLE_NAME 
																		+ " (" + "_id"+ " INTEGER PRIMARY KEY AUTOINCREMENT, " 
																		+ UTLog.FIELD_NAME_EVENTID + " TEXT,"
																		+ UTLog.FIELD_NAME_PRIORITY + " TEXT, " 
																		+ UTLog.FIELD_NAME_STREAMID + " TEXT, "
																		+ UTLog.FIELD_NAME_TIME + " TEXT, " 
																		+ UTLog.FIELD_NAME_INDEX + " TEXT, "
																		+ UTLog.FIELD_NAME_CONTENT + " TEXT " + ")";
	String						querySql						= "SELECT * FROM %s ORDER BY %s DESC, %s DESC LIMIT %d";
	String						countSql						= "SELECT count(*) FROM %s";
	SqliteHelper				mDbHelper;

	protected UTLogSqliteStore(Context context) {
		mDbHelper = new SqliteHelper(context);
	}

	public synchronized boolean insert(List<UTLog> logs) {
		if (logs == null || logs.size() == 0) {
			return true;
		} else {
			SQLiteDatabase db = mDbHelper.getWritableDatabase();
			boolean isSuccess = false;
			try {
				if (db != null ) {
					db.beginTransaction();
					isSuccess = true;
					for (int i = 0; i < logs.size(); i++) {
						UTLog log = logs.get(i);
						if (log != null) {
							ContentValues values = new ContentValues();
							values.put(UTLog.FIELD_NAME_EVENTID, log.eventId);
							values.put(UTLog.FIELD_NAME_PRIORITY, log.priority);
							String content = log.getCipherContent();
							values.put(UTLog.FIELD_NAME_CONTENT, content);
							values.put(UTLog.FIELD_NAME_TIME, log.time);
							values.put(UTLog.FIELD_NAME_INDEX, log.key);
							db.insert(TABLE_NAME, "", values);
						}
						UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "insert log :" + log + " isSuccess:" + isSuccess);
					}
					db.setTransactionSuccessful();
					db.endTransaction();
				} else {
					UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "db is null");
				}
			} catch(Throwable e){
				UTMCLogger.w(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[insert] " + e);
			}
			finally {
				mDbHelper.closeWritableDatabase(db);
			}
			UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[insert] " + logs.size()+" logsize:"+count());
			return isSuccess;
		}
	}

	public synchronized boolean delete(List<UTLog> logs) {
		if (logs == null || logs.size() == 0) {
			return true;
		}
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		boolean isSuccess = false;
		if (db != null) {
			try {
				db.beginTransaction();
				isSuccess = true;
				for (int i = 0; i < logs.size(); i++) {
					long ret = db.delete(TABLE_NAME, "_id=?", new String[] { logs.get(i).id + "" });
					if (ret <= 0) {
						isSuccess = false;
						UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "delete  " + logs.get(i) + " ret:" + ret);
					}
				}
			}catch(Throwable e){
				UTMCLogger.w(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[delete] " + e);
			}finally {
				db.setTransactionSuccessful();
				db.endTransaction();
				mDbHelper.closeWritableDatabase(db);
			}
		} else {
			UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "db is null");
		}
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "delete " + logs.size() + " isSuccess:" + isSuccess);
		return isSuccess;
	}

	public synchronized ArrayList<UTLog> get(int maxCount) {
		if (maxCount <= 0) {
			return null;
		}
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		ArrayList<UTLog> logs = new ArrayList<UTLog>();
		if (db != null) {
			String sql = String.format(querySql, TABLE_NAME, UTLog.FIELD_NAME_TIME, UTLog.FIELD_NAME_ID, maxCount);
			Cursor c = db.rawQuery(sql, null);
			try {
				while (c.moveToNext()) {
					UTLog log = new UTLog();
					log.id = c.getInt(c.getColumnIndex(UTLog.FIELD_NAME_ID));
					log.eventId = c.getString(c.getColumnIndex(UTLog.FIELD_NAME_EVENTID));
					log.priority = c.getString(c.getColumnIndex(UTLog.FIELD_NAME_PRIORITY));
					String content = c.getString(c.getColumnIndex(UTLog.FIELD_NAME_CONTENT));
					log.setCipherContent(content);
					log.time = c.getString(c.getColumnIndex(UTLog.FIELD_NAME_TIME));
					log.key = c.getString(c.getColumnIndex(UTLog.FIELD_NAME_INDEX));
					logs.add(log);
				}
			} catch (Exception e) {
				UTMCLogger.w(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[get] " + e);
			} finally {
				if (c != null) {
					c.close();
				}
				mDbHelper.closeWritableDatabase(db);
			}

		} else {
			UTMCLogger.e(UTMCLogger.LOG_TYPE_DEBUG, TAG, "db is null");
		}
		return logs;
	}

	public synchronized int count() {
		int count = 0;
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		try {
			if (db != null) {
				String sql = String.format(countSql, TABLE_NAME);
				Cursor cursor = db.rawQuery(sql, null);
				if (cursor != null) {
					cursor.moveToFirst();
					count = cursor.getInt(0);
					cursor.close();
				}
			} else {
				UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "db is null");
			}
		} catch (Throwable e) {
		}finally{
			mDbHelper.closeWritableDatabase(db);
		}
		return count;
	}

	@Override
	public synchronized void clear() {
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		if (db != null) {
			db.delete(TABLE_NAME, null, null);
			mDbHelper.closeWritableDatabase(db);
		}
	}

	public boolean isAvailable() {
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		if (db == null) {
			return false;
		} else {
			mDbHelper.closeWritableDatabase(db);
			return true;
		}
	}

	class SqliteHelper extends SQLiteOpenHelper {
//		private AtomicInteger	mWritableCounter	= new AtomicInteger();
		private SQLiteDatabase	mWritableDatabase;

		SqliteHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(CREATE_TABLE_IF_NO_EXIST_SQL);
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		}

		@SuppressLint("NewApi")
		@Override
		public synchronized SQLiteDatabase getWritableDatabase() {
			try {
				if (mWritableDatabase == null) {
					mWritableDatabase = super.getWritableDatabase();
					
				}
				if (mWritableDatabase!=null) {
					mWritableDatabase.acquireReference();
					if(Build.VERSION.SDK_INT >= 11){
						mWritableDatabase.enableWriteAheadLogging();
					}
				}
			} catch (Throwable e) {
				UTMCLogger.w(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[get db] " + e);
			}
			return mWritableDatabase;
		}

		public synchronized void closeWritableDatabase(SQLiteDatabase db) {
			if (db == null) {
				return;
			}
			if (mWritableDatabase != null) {
				mWritableDatabase.releaseReference();
			}
		}
	}

}