package com.ut.store;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import com.ut.mini.base.UTMCStatConfig;
import com.ut.mini.log.UTMCLogger;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;

public class UTLogStoreMgr {

	private static final long STORE_INTERVAL = 5 * 1000;
	private static final int MAX_LOG_SIZE = 100;
	private static final int MSG_STORE = 100;
	private static final String TAG = UTLogStoreMgr.class.getSimpleName();
	private static UTLogStoreMgr mInstance;

	private IUTLogStore mStore;
	private List<UTLog> mLogs = new CopyOnWriteArrayList<UTLog>();
	HandlerThread mThread;
	Handler mHandler;

	Runnable mStoreTask = new Runnable() {
		@Override
		public void run() {
			store();
		}
	};

	private UTLogStoreMgr() {
		mStore = new UTLogSqliteStore(UTMCStatConfig.getInstance().getContext());
		mThread = new HandlerThread("log stroe");
		mThread.start();
		mHandler = new Handler(mThread.getLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (msg.what == MSG_STORE) {
					store();
				}
				super.handleMessage(msg);
			}
		};
	}

	public synchronized static UTLogStoreMgr getInstance() {
		if (mInstance == null) {
			mInstance = new UTLogStoreMgr();
		}
		return mInstance;
	}

	public void add(UTLog log) {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[add] eventId:"+log.eventId);
		
		log.time = System.currentTimeMillis() + "";
		mLogs.add(log);
		if (mLogs.size() >= MAX_LOG_SIZE) {
			mHandler.removeMessages(MSG_STORE);
			mHandler.sendEmptyMessage(MSG_STORE);
		} else if (!mHandler.hasMessages(MSG_STORE)) {
			mHandler.sendEmptyMessageDelayed(MSG_STORE, STORE_INTERVAL);
		}
	}

	
	public boolean delete(List<UTLog> logs) {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[delete]" + " logs.size():" + logs.size());
		return mStore.delete(logs);

	}

	public List<UTLog> get(int maxCount) {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[get]" + " maxCount:" + maxCount);
		store();
		return mStore.get(maxCount);
		//return new ArrayList<UTLog>();
	}


	public void store() {
		List<UTLog> temp =null;
		synchronized (mLogs) {
			if (mLogs.size() > 0) {
				temp = new ArrayList<UTLog>(mLogs);
				mLogs.clear();
			}
		}
		if (temp != null && temp.size() > 0) {
			mStore.insert(temp);
		}
	}

	public void clear() {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[clear]");
		mStore.clear();
		mLogs.clear();
	}

	public long count() {
		UTMCLogger.d(UTMCLogger.LOG_TYPE_DEBUG, TAG, "[count] memory count:"+mLogs.size()+" db count:"+mStore.count());
		return mStore.count() + mLogs.size();
	}
	public long memoryCount(){
		return mLogs.size();
	}
	public long dbCount(){
		return mStore.count();
	}
}
